import 'dart:convert';
import 'dart:developer';

import 'package:dio/dio.dart' as Dio;
import 'package:get/get.dart';

const int _SUCCESS = 200;
const int _ERROR_TOKEN = 401;
const int _ERROR_VALIDATE = 422;
const int _ERROR_SERVER = 500;
const int _ERROR_DISCONNECT = -1;

class NetworkResponse<T> {
  int? code;
  T? data;
  dynamic error;

  bool get isSuccess => code == _SUCCESS && data != null;

  bool get isError => code != _SUCCESS;

  NetworkResponse({this.data, this.code});

  factory NetworkResponse.fromResponse(Dio.Response response,
      {converter, value, String? prefix}) {
    try {
      return NetworkResponse._fromJson(response.data,
          converter: converter, prefix: prefix, value: value)
        ..code = response.statusCode;
    } catch (e) {
      log("Error NetworkResponse.fromResponse: $e");
      return NetworkResponse.withErrorConvert(e);
    }
  }

  NetworkResponse._fromJson(dynamic json, {converter, value, String? prefix}) {
    if (value != null)
      data = value;
    else if (prefix != null) {
      if (prefix.trim().isEmpty)
        data = converter != null && json != null ? converter(json) : json;
      else
        data = converter != null && json[prefix] != null
            ? converter(json[prefix])
            : json[prefix];
    } else {
      data = converter != null && json["values"] != null
          ? converter(json["values"])
          : json["values"];
    }
  }

  Map<String, dynamic> toJson() {
    var map = <String, dynamic>{};
    map["code"] = code;
    map["data"] = data;
    return map;
  }

  NetworkResponse.withErrorRequest(Dio.DioError error) {
    try {
      this.data = null;
      Dio.Response? response = error.response;
      this.code = response?.statusCode ?? _ERROR_SERVER;
      if (this.code == 400 || this.code == 401)
        this.error = response?.data?['error'] ?? 'unknown error';
    } catch (e) {
      log("Error NetworkResponse.withErrorRequest: $e");
    }
  }

  NetworkResponse.withErrorConvert(error) {
    this.data = null;
    this.error = 'unknown error';
  }

  NetworkResponse.withDisconnect() {
    this.data = null;
  }
}
