import '../model/model.dart';

abstract class AppApi {
  /// Auth
  Future<NetworkResponse> refreshUserToken(refreshToken);

  Future<NetworkResponse> refreshProToken(refreshToken);

  Future<NetworkResponse> checkNewPhoneFromUser(phone);
  Future<NetworkResponse> checkNewPhoneFromPro(phone);

  Future<NetworkResponse> checkExistPhoneFromUser(phone);

  Future<NetworkResponse> createOrLoginPro(params);

  Future<NetworkResponse> loginUserByPhone(params);

  Future<NetworkResponse> loginUserByEmail(params);

  Future<NetworkResponse> createOrLoginUser(params);

  Future<NetworkResponse> createNewUser(params);
  Future<NetworkResponse> subRegisterUser(params);

  Future<NetworkResponse> getUserInfo();

  Future<NetworkResponse> getProInfo();

  Future<NetworkResponse> updatePro(params);

  Future<NetworkResponse> updateUser(params);

  Future<NetworkResponse> updateNewUserPasswordByFirebase(params);

  Future<NetworkResponse> addNotificationToken(params);

  Future<NetworkResponse> setThumbnailForUser(image, filePath);

  Future<NetworkResponse> setThumbnailForPro(image, filePath);

  Future<NetworkResponse> updateProPwd(String oldPassword, String newPassword);

  Future<NetworkResponse> logoutPro([fcmToken]);

  Future<NetworkResponse> logoutUser([fcmToken]);

  Future<NetworkResponse> checkUpdateEmailFromUser(params);

  Future<NetworkResponse> confirmUpdateUserEmailCode(email, code, phone);

  Future<NetworkResponse> updateUserEmail(token);

  Future<NetworkResponse> checkUpdateEmailFromPro(params);

  Future<NetworkResponse> confirmUpdateProEmailCode(email, code );

  Future<NetworkResponse> updateProEmail(token);

  Future<NetworkResponse> loginPro(params);
  Future<NetworkResponse> loginUserByPhoneAndFirebase(params);
  Future<NetworkResponse> loginProByPhoneAndFirebase(params);

  Future<NetworkResponse> loginProCrmByQrCode(params);

  Future<NetworkResponse> checkNewProEmail(params);
  Future<NetworkResponse> checkEmailPro(params);

  Future<NetworkResponse> confirmNewProCode(params);

  Future<NetworkResponse> createNewPro(params);

  Future<NetworkResponse> getCategoriesList();

  Future<NetworkResponse> checkExistEmailFromPro(params);

  Future<NetworkResponse> resetProPassword(params);

  Future<NetworkResponse> confirmResetProPasswordCode(params);

  Future<NetworkResponse> updateNewProPasswordByToken(params);

  Future<NetworkResponse> getProInfoByUniqueId(uniqueId);
  Future<NetworkResponse> getAssetsForProFromUniversalDashboard(proUniqueId);
  Future<NetworkResponse> getIndustriesList();
  Future<NetworkResponse> checkLoginProByEmailOTP(params);
  Future<NetworkResponse> loginProByEmailOTPConfirm(params);

  ///Google place
  Future<NetworkResponse> googlePlaceAutoComplete(
      String keyword, String countryCode,
      [String? address]);

  Future<NetworkResponse> googlePlaceSearchById(String placeId);

  Future<NetworkResponse> googlePlaceSeachPlaceByText(String keyword,
      [String? types]);

  Future<NetworkResponse> googlePlaceSeachPlaceDetailById(placeId);

  Future<NetworkResponse> googleGeocode(latlng);

  /// Core
  Future<NetworkResponse> getDateFormatList();
  Future<NetworkResponse> getTimeFormatList();
  Future<NetworkResponse> getPhoneAvailable();

  Future<NetworkResponse> getCountryAvailable();

  Future<NetworkResponse> getLanguageAvailable();

  Future<NetworkResponse> getLanguageForSpeakAvailable();

  Future<NetworkResponse> getAllowedFeaturesForOowooApp();

  Future<NetworkResponse> getTncOowooApp();

  Future<NetworkResponse> getTncOowooSeller();

  Future<NetworkResponse> testSendNotificationToUser(params);

  Future<NetworkResponse> testSendNotificationToPro(params);

  Future<NetworkResponse> notifVideoCallUserFromPro(params);

  Future<NetworkResponse> notifVideoCallProFromUser(params);
  Future<NetworkResponse> getDashboardInfos();

  ///Contacts
  Future<NetworkResponse> getContactsList(params);

  Future<NetworkResponse> addUserInContactsList(params);

  Future<NetworkResponse> updateCommentForContact(params);

  Future<NetworkResponse> deleteContact(params);

  Future<NetworkResponse> getExistingUserByPhone(params);
  Future<NetworkResponse> checkAndAddNewContact(params);

  ///My products
  Future<NetworkResponse> getProductCategories(); 
  Future<NetworkResponse> getStatusList(); 
  Future<NetworkResponse> getProductsFromPro(params); 
  Future<NetworkResponse> updateTerminalFromPro(params);
  Future<NetworkResponse> sendPropertyToTerminal(params);
  Future<NetworkResponse> removePropertyFromTerminal(params);

  Future<NetworkResponse> updateProduct(params);

  Future<NetworkResponse> getTerminalsFromPro(terminalType);
  Future<NetworkResponse> linkTerminalToProV2(params);

  ///PublicEvents
  Future<NetworkResponse> getAllPublicEventsAroundMe(params);

  Future<NetworkResponse> getPublicEventsListFromUser(params);

  Future<NetworkResponse> getPublicEventsCategories(languageCode);

  Future<NetworkResponse> getFavoritePublicEventsCategoriesFromUser(
      languageCode);

  Future<NetworkResponse> getSuggestionEventsFromUser(params);

  Future<NetworkResponse> getAllPublicEventsFromSearch(params);

  Future<NetworkResponse> getLikedPublicEventsFromUser();

  Future<NetworkResponse> addUserLikeToPublicEvent(publicEventId);

  Future<NetworkResponse> setFavoriteCatForUser(catId);

  Future<NetworkResponse> removeFavoriteCatForUser(catId);

  Future<NetworkResponse> addSeenSuggestionEventFromUser(
      publicEventId, utcModifier);

  Future<NetworkResponse> bookFreeTicketForPublicEvents(proUniqueId);

  Future<NetworkResponse> getTicketListFromPublicEvent(publicEventId);

  ///Payments
  Future<NetworkResponse> getCustomerCardsStripe(page, pageSize);

  Future<NetworkResponse> createCustomerCardStripe(params);

  Future<NetworkResponse> deleteCustomerCardStripe(cardId);

  Future<NetworkResponse> setCustomerCardDefaultStripe(cardId);

  Future<NetworkResponse> getWalletInfo();

  Future<NetworkResponse> createPurchaseByCardOnFileStripe(params);

  ///Sellers
  Future<NetworkResponse> becomeSeller(confirmed);

  Future<NetworkResponse> updateImageForBecomeSeller(fileType, file);

  Future<NetworkResponse> quitSellerProgram(temp);

  Future<NetworkResponse> recoverySellerProgram();

  Future<NetworkResponse> getMissionsFromSeller(params);

  Future<NetworkResponse> getMissionTypes(params);

  Future<NetworkResponse> takeMissionFromSeller(missionId);

  Future<NetworkResponse> getNumberOfSalesFromSellers(params);

  Future<NetworkResponse> setBankAccountFromUser(params);

  Future<NetworkResponse> getAccountTypes();

  Future<NetworkResponse> getBankInfosFromSeller();

  Future<NetworkResponse> updateBankInfosFromSeller(params);

  ///Other
  // https://byabbe.se/on-this-day/3/18/events.json
  // https://byabbe.se/on-this-day/3/18/births.json
  // https://byabbe.se/on-this-day/3/18/deaths.json
  Future<NetworkResponse> eventsOnThisDay(date);

  Future<NetworkResponse> birthsOnThisDay(date);

  Future<NetworkResponse> deathsOnThisDay(date);

  Future<NetworkResponse> pexelsSearch(params);

  //Weather
  Future<NetworkResponse> weatherForecast(params);

  Future<NetworkResponse> weatherHistory(params);

  //Bars restaurants
  Future<NetworkResponse> getOoBRPlacesAroundMe(params);

  Future<NetworkResponse> getOoBRPlacesByFilterValues(params);

  Future<NetworkResponse> getFavoriteOobrPlaces();

  Future<NetworkResponse> getOoBRPlaceInfoById(params);

  Future<NetworkResponse> updateFilterValueForOobr(params);

  Future<NetworkResponse> getPromotedVideosForBar(oobrPlaceId);

  Future<NetworkResponse> getPromotedVideosForOobrPlaces(lat, lng);

  Future<NetworkResponse> getOoBRCategories(languageCode);

  Future<NetworkResponse> getOoBRFilters(languageCode);

  Future<NetworkResponse> checkBarGooglePlaceIdByContributor(googlePlaceId);

  Future<NetworkResponse> createNewOobrPlaceByContributor(params);

  Future<NetworkResponse> addOobrPlaceToFavorites(oobrPlaceId);

  Future<NetworkResponse> getOobrPlaceSchedules(oobrPlaceId);

  Future<NetworkResponse> getOobrPlaceMenuList(oobrPlaceId, languageCode);

  Future<NetworkResponse> getOobrPlaceMenuContent(menuId, languageCode);

  Future<NetworkResponse> getOptionsForItem(itemId, languageCode);

  Future<NetworkResponse> getAllergensForItem(itemId, languageCode);

  Future<NetworkResponse> createOrderForOobrPlace(params);

  Future<NetworkResponse> getAnythingFromQr(params);

  Future<NetworkResponse> getDetailOobrInvitations(oobrEventId);

  Future<NetworkResponse> inviteContactsToOobrPlace(params);

  Future<NetworkResponse> answerOobrInvitation(params);

  ///Emergency
  Future<NetworkResponse> getEmergencyInfos(params);

  Future<NetworkResponse> sendEmergencyRequest(params);

  ///PersonalEvents
  Future<NetworkResponse> createSharedPcEvent(params);

  Future<NetworkResponse> setThumbnailForPcEvent(path, pcEventId);

  Future<NetworkResponse> shareCalendarToUsers(params);

  Future<NetworkResponse> getPcEventFromUser(pcEventId);

  Future<NetworkResponse> getPcEventFromPro(pcEventId);

  ///Home
  Future<NetworkResponse> getHomesAroundMe(params);

  Future<NetworkResponse> getPromotedVideosForOoh(lat, lng);

  ///Whatnext
  Future<NetworkResponse> whatsNext(params);
  Future<NetworkResponse> getReminderTypes();

  //Agora
  Future<NetworkResponse> generateToken(channelName, expiredTime);

  //Homelido
  Future<NetworkResponse> getHomelidoPropertyCategories(languageCode);

  Future<NetworkResponse> getPropertiesFromPro(languageCode);

  Future<NetworkResponse> getCurrentPropertiesOfTerminal(params);

  Future<NetworkResponse> createHomlidoMeeting(params);

  Future<NetworkResponse> getHomelidoMeetingDetails(params);

  Future<NetworkResponse> answerToHomelidoVisitRequest(params);

  //Tracking position
  Future<NetworkResponse> getTrackingInfosForMeeting(params);

  Future<NetworkResponse> addUserOrProToTrackingPeople(params);

  Future<NetworkResponse> updateTrackingInfosForMeeting(params);

  Future<NetworkResponse> getLearnMoreTrackingVideo();

  //Move
  Future<NetworkResponse> getOowooMoveProperties(params);

  Future<NetworkResponse> getOowooVideoByTypeId(typeId);

  //Relax
  Future<NetworkResponse> getCalmCategories();

  Future<NetworkResponse> getCalmByCat(params);

  Future<NetworkResponse> subscribeToCalmId(calmId);

  Future<NetworkResponse> getSubscribersForCalmId(params);

  //Agora
  Future<NetworkResponse> getReviewsFromPro(proId);

  //Appointment
  Future<NetworkResponse> getAllEventsFromUser();
  Future<NetworkResponse> getEventInfoFromUser(eventUniqueId);
  Future<NetworkResponse> getQueueInfoFromUser(ticketUniqueId);
  Future<NetworkResponse> cancelTicketByUser(ticketUniqueId, status, comment);
  Future<NetworkResponse> confirmTicketByUser(ticketUniqueId, status, comment);
  Future<NetworkResponse> ticketUserCallBack(ticketUniqueId, duration);
  Future<NetworkResponse> deleteEventsFromUser(ticketUniqueId);
  Future<NetworkResponse> getUserInfoByFirebaseId(firebaseId);
  Future<NetworkResponse> getProInfoByFirebaseId(firebaseId);

  //Others
  Future<NetworkResponse> getVideoConfigFromPlayerVimeo(vimeoId);

  Future<NetworkResponse> getFavouriteForUser();
  Future<NetworkResponse> removeFromFavourite(proId);
  Future<NetworkResponse> addToFavourite(proId);

  //Digicodes
  Future<NetworkResponse> getCodesFromPro(digicodeId);
  Future<NetworkResponse> createCode(params);
  Future<NetworkResponse> updateLiveCode(params);
  Future<NetworkResponse> shareCodeDigicode(params);
  Future<NetworkResponse> deleteLiveCode(params);
  Future<NetworkResponse> getDigicodesFromPro();
  Future<NetworkResponse> getPlacesFromPro(params);
  Future<NetworkResponse> unlockDigicode(params);
  Future<NetworkResponse> getDigicodesLog(params);
  Future<NetworkResponse> deleteCodeFromLog(params);
  Future<NetworkResponse> checkDigicodeOnline(digicodeId);
  Future<NetworkResponse> getDigicodeById(id);
  Future<NetworkResponse> checkDigicodeBySerialAndToken(
      productSerial, authcode);
  Future<NetworkResponse> linkDigicodeToPro(params);
  Future<NetworkResponse> linkDigicodeToProV2(params);

  //Pro zone
  Future<NetworkResponse> setMainLanguageForPro(proLanguageCode);

  Future<NetworkResponse> setLanguagesForPro(proLanguageCode);

  // Appointment
  Future<NetworkResponse> getAllEventsFromPro();
  Future<NetworkResponse> getAllServicesOfProFromPro(params);
  Future<NetworkResponse> updateDelay(params);
  Future<NetworkResponse> cancelTicketByPro(ticketUniqueId);
  Future<NetworkResponse> confirmTicketByPro(ticketUniqueId, status, comment);
  Future<NetworkResponse> declareUserAsAbsentFromPro(ticketUniqueId);
  Future<NetworkResponse> ticketUserNotResponse(ticketUniqueId);
  Future<NetworkResponse> startTicket(ticketUniqueId);
  Future<NetworkResponse> stopTicket(ticketUniqueId);
  Future<NetworkResponse> canceledByProAfterCallBack(ticketUniqueId);
  Future<NetworkResponse> callNextTicket(serviceId);
  Future<NetworkResponse> getEventInfoFromPro(eventUniqueId);
  Future<NetworkResponse> getListTimeFramesByServiceForDay(
      dateToday, serviceId);
  Future<NetworkResponse> getServiceAvailabilityForMonth(params);
  Future<NetworkResponse> createTicketByPro(params);
  Future<NetworkResponse> deleteEventsFromPro(ticketUniqueId);
  Future<NetworkResponse> createTicketByUser(params);
  Future<NetworkResponse> getAllServicesOfProFromUser(params);
  Future<NetworkResponse> getAllServicesAroundMe(params);

  //Playlist
  // Future<NetworkResponse> v2_getPlaylistFromPro(params);
  // Future<NetworkResponse> v2_createPlaylistFromPro(params);
  // Future<NetworkResponse> v2_deletePlaylistFromPro(params);
  // Future<NetworkResponse> v2_getContentItemsFromPro(params);
  // Future<NetworkResponse> v2_addItemToPlaylist(params);
  // Future<NetworkResponse> v2_removeItemFromPlaylist(params);
  // Future<NetworkResponse> v2_getPlaylistsFromTerminal(params);

  //Security
  Future<NetworkResponse> getLoginSessions(params);
  Future<NetworkResponse> revokeLoginSession(params);
  Future<NetworkResponse> allowPendingLoginOnPlatformByQrCode(params);
  Future<NetworkResponse> allowPendingLoginOnPlatformById(params); 

  ///History pmb
  Future<NetworkResponse> getHistoryFromUser();

  ///pmb
  Future<NetworkResponse> getBgVideo(countryCode);

  Future<NetworkResponse> getPmbDevicesAroundMe(params);

  Future<NetworkResponse> scanDeviceQr(params);

  Future<NetworkResponse> activatePourDevice(deviceId);

  Future<NetworkResponse> ipAPI();

  Future<NetworkResponse> getCurrencyAvailableForToken(tokenType);

  Future<NetworkResponse> getCurrentTokenValue(tokenType, {currency = 'EUR'});
}
