import 'dart:io';

import 'package:dio/dio.dart';
import 'package:oowoo_core/oowoo_core.dart';
import '../model/google_place/autocomplete_prediction.dart';
import '../model/google_place/details_response.dart';
import '../model/terminal.dart';
import '../repository/agora_repo.dart';
import 'app_endpoint.dart';
import '../resources.dart';

class AppApiImp extends AppApi {
  @override
  Future<NetworkResponse> refreshUserToken(refreshToken) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response =
          await AppClients(requiredToken: true, token: refreshToken)
              .post(AppEndpoint.refreshUserToken());
      return NetworkResponse.fromResponse(response,
          converter: (json) => Token.fromJson(json), prefix: '');
    } on DioError catch (e) {
      return NetworkResponse.withErrorRequest(e);
    }
  }

  @override
  Future<NetworkResponse> refreshProToken(refreshToken) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response =
          await AppClients(requiredToken: true, token: refreshToken)
              .post(AppEndpoint.refreshProToken());
      return NetworkResponse.fromResponse(response,
          converter: (json) => Token.fromJson(json), prefix: '');
    } on DioError catch (e) {
      return NetworkResponse.withErrorRequest(e);
    }
  }

  @override
  Future<NetworkResponse> checkNewPhoneFromUser(phone) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response =
          await AppClients(requiredToken: false, enableErrorHandler: false).get(
              AppEndpoint.checkNewPhoneFromUser(),
              queryParameters: {'phone': phone});
      return NetworkResponse.fromResponse(response,
          value: response.statusCode == 200);
    } on DioError catch (e) {
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> checkNewPhoneFromPro(phone) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response =
          await AppClients(requiredToken: false, enableErrorHandler: false).get(
              AppEndpoint.checkNewPhoneFromPro(),
              queryParameters: {'phone': phone});
      return NetworkResponse.fromResponse(response,
          value: response.statusCode == 200);
    } on DioError catch (e) {
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> checkExistPhoneFromUser(phone) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClients(requiredToken: false).get(
          AppEndpoint.checkExistPhoneFromUser(),
          queryParameters: {'phone': phone});
      return NetworkResponse.fromResponse(response,
          converter: (json) => UserModel.fromJson(json), prefix: '');
    } on DioError catch (e) {
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> addNotificationToken(params) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClients(
              requiredToken: true, token: await appPrefs.getNormalToken())
          .post(AppEndpoint.addNotificationToken(), data: params);
      return NetworkResponse.fromResponse(response,
          value: response.statusCode == 200);
    } on DioError catch (e) {
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> createOrLoginPro(params) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClients(requiredToken: false)
          .post(AppEndpoint.createOrLoginPro(), data: params);
      return NetworkResponse.fromResponse(response,
          converter: (json) => Token.fromJson(json), prefix: '');
    } on DioError catch (e) {
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      appDebugPrint('Error: $e');
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> updateNewUserPasswordByFirebase(params) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClients(requiredToken: false)
          .post(AppEndpoint.updateNewUserPasswordByFirebase(), data: params);
      return NetworkResponse.fromResponse(response,
          value: response.statusCode == 200);
    } on DioError catch (e) {
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      appDebugPrint('Error: $e');
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> loginUserByPhone(params) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClients(
        requiredToken: false,
        enableErrorHandler: false,
      ).post(
        AppEndpoint.loginUserByPhone(),
        data: params,
      );
      return NetworkResponse.fromResponse(response,
          converter: (json) => Token.fromJson(json), prefix: '');
    } on DioError catch (e) {
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      appDebugPrint('Error: $e');
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> loginUserByPhoneAndFirebase(params) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClients(
        requiredToken: false,
        enableErrorHandler: false,
      ).post(
        AppEndpoint.loginUserByPhoneAndFirebase(),
        data: params,
      );
      return NetworkResponse.fromResponse(response,
          converter: (json) => Token.fromJson(json), prefix: '');
    } on DioError catch (e) {
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      appDebugPrint('Error: $e');
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> loginUserByEmail(params) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClients(requiredToken: false)
          .post(AppEndpoint.loginUserByEmail(), data: params);
      return NetworkResponse.fromResponse(response,
          converter: (json) => Token.fromJson(json), prefix: '');
    } on DioError catch (e) {
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      appDebugPrint('Error: $e');
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> createOrLoginUser(params) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClients(requiredToken: false)
          .post(AppEndpoint.createOrLoginUser(), data: params);
      return NetworkResponse.fromResponse(response,
          converter: (json) => Token.fromJson(json), prefix: '');
    } on DioError catch (e) {
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      appDebugPrint('Error: $e');
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> createNewUser(params) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClients(requiredToken: false)
          .post(AppEndpoint.createNewUser(), data: params);
      return NetworkResponse.fromResponse(response,
          converter: (json) => Token.fromJson(json), prefix: '');
    } on DioError catch (e) {
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      appDebugPrint('Error: $e');
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> subRegisterUser(params) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      final token = await appPrefs.getNormalToken();
      Response response = await AppClients(requiredToken: true, token: token)
          .post(AppEndpoint.subRegisterUser(), data: params);
      return NetworkResponse.fromResponse(response);
    } on DioError catch (e) {
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      appDebugPrint('Error: $e');
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> getUserInfo() async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      final token = await appPrefs.getNormalToken();
      Response response = await AppClients(requiredToken: true, token: token)
          .get(AppEndpoint.getUserInfo());
      return NetworkResponse.fromResponse(response,
          converter: (json) => UserModel.fromJson(json));
    } on DioError catch (e) {
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      appDebugPrint('Error: $e');
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> getProInfo() async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      final token = await appPrefs.getNormalToken();
      Response response = await AppClients(requiredToken: true, token: token)
          .get(AppEndpoint.getProInfo());
      return NetworkResponse.fromResponse(response,
          converter: (json) => ProInfo.fromJson(json));
    } on DioError catch (e) {
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      appDebugPrint('Error: $e');
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> updateUser(params) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      final token = await appPrefs.getNormalToken();
      Response response = await AppClients(requiredToken: true, token: token)
          .put(AppEndpoint.updateUser(), data: params);
      return NetworkResponse.fromResponse(response,
          value: UpdateInfoResult(status: response.statusCode == 200));
    } on DioError catch (e) {
      if (e.response != null &&
          e.response!.statusCode == 400 &&
          e.response!.data != null) {
        return NetworkResponse.fromResponse(e.response!,
            value: UpdateInfoResult(
                status: false, msg: e.response!.data['error']));
      }
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      appDebugPrint('Error: $e');
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> updatePro(params) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      final token = await appPrefs.getNormalToken();
      Response response = await AppClients(requiredToken: true, token: token)
          .put(AppEndpoint.updatePro(), data: params);
      return NetworkResponse.fromResponse(response,
          value: response.statusCode == 200);
    } on DioError catch (e) {
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      appDebugPrint('Error: $e');
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> logoutPro([fcmToken]) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      final token = await appPrefs.getNormalToken();
      Response response = await AppClients(requiredToken: true, token: token)
          .post(AppEndpoint.logoutPro(), data: {'fcmToken': fcmToken});
      return NetworkResponse.fromResponse(response,
          value: response.statusCode == 200);
    } on DioError catch (e) {
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      appDebugPrint('Error: $e');
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> logoutUser([fcmToken]) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      final token = await appPrefs.getNormalToken();
      Response response = await AppClients(requiredToken: true, token: token)
          .post(AppEndpoint.logoutUser(), data: {'fcmToken': fcmToken});
      return NetworkResponse.fromResponse(response,
          value: response.statusCode == 200);
    } on DioError catch (e) {
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      appDebugPrint('Error: $e');
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> setThumbnailForUser(image, filePath) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      String fileName = filePath.split('/').last;
      FormData formData = FormData.fromMap({
        "fileName": MultipartFile.fromBytes(image, filename: fileName),
      });
      final token = await appPrefs.getNormalToken();
      Response response = await AppClients(requiredToken: true, token: token)
          .post(AppEndpoint.setThumbnailForUser(), data: formData);
      return NetworkResponse.fromResponse(response,
          converter: (json) => json['fileName']);
    } on DioError catch (e) {
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      appDebugPrint('Error: $e');
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> setThumbnailForPro(image, filePath) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      String fileName = filePath.split('/').last;
      FormData formData = FormData.fromMap({
        "fileName": MultipartFile.fromBytes(image, filename: fileName),
      });
      final token = await appPrefs.getNormalToken();
      Response response = await AppClients(requiredToken: true, token: token)
          .post(AppEndpoint.setThumbnailForPro(), data: formData);
      return NetworkResponse.fromResponse(response,
          converter: (json) => json['fileName']);
    } on DioError catch (e) {
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      appDebugPrint('Error: $e');
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> updateProPwd(
      String oldPassword, String newPassword) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClients(requiredToken: true).put(
          AppEndpoint.updateProPwd(),
          data: {'oldPassword': oldPassword, 'newPassword': newPassword});
      return NetworkResponse.fromResponse(response,
          value: response.statusCode == 200);
    } on DioError catch (e) {
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      appDebugPrint('Error: $e');
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> checkUpdateEmailFromUser(params) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      final token = await appPrefs.getNormalToken();
      Response response = await AppClients(requiredToken: true, token: token)
          .post(AppEndpoint.checkUpdateEmailFromUser(), data: params);
      return NetworkResponse.fromResponse(response,
          value: UpdateInfoResult(status: response.statusCode == 200));
    } on DioError catch (e) {
      if (e.response != null &&
          e.response!.statusCode == 400 &&
          e.response!.data != null) {
        return NetworkResponse.fromResponse(e.response!,
            value: UpdateInfoResult(
                status: false, msg: e.response!.data['error']));
      }
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      appDebugPrint('Error: $e');
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> confirmUpdateUserEmailCode(email, code, phone) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      final token = await appPrefs.getNormalToken();
      Response response = await AppClients(requiredToken: true, token: token)
          .post(AppEndpoint.confirmUpdateUserEmailCode(),
              data: {'email': email, 'code': code, 'phone': phone});
      return NetworkResponse.fromResponse(response,
          converter: (json) => ConfirmEmailCodeResponse.fromJson(json));
    } on DioError catch (e) {
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      appDebugPrint('Error: $e');
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> updateUserEmail(_token) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      final token = await appPrefs.getNormalToken();
      Response response = await AppClients(requiredToken: true, token: token)
          .put(AppEndpoint.updateUserEmail(), data: {'token': _token});
      return NetworkResponse.fromResponse(response,
          value: response.statusCode == 200);
    } on DioError catch (e) {
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      appDebugPrint('Error: $e');
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> checkUpdateEmailFromPro(params) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      final token = await appPrefs.getNormalToken();
      Response response = await AppClients(requiredToken: true, token: token)
          .post(AppEndpoint.checkUpdateEmailFromPro(), data: params);
      return NetworkResponse.fromResponse(response,
          value: UpdateInfoResult(status: response.statusCode == 200));
    } on DioError catch (e) {
      if (e.response != null &&
          e.response!.statusCode == 400 &&
          e.response!.data != null) {
        return NetworkResponse.fromResponse(e.response!,
            value: UpdateInfoResult(
                status: false, msg: e.response!.data['error']));
      }
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      appDebugPrint('Error: $e');
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> confirmUpdateProEmailCode(email, code) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      final token = await appPrefs.getNormalToken();
      Response response = await AppClients(requiredToken: true, token: token)
          .post(AppEndpoint.confirmUpdateProEmailCode(),
              data: {'email': email, 'code': code});
      return NetworkResponse.fromResponse(response,
          converter: (json) => ConfirmEmailCodeResponse.fromJson(json));
    } on DioError catch (e) {
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      appDebugPrint('Error: $e');
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> updateProEmail(_token) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      final token = await appPrefs.getNormalToken();
      Response response = await AppClients(requiredToken: true, token: token)
          .put(AppEndpoint.updateProEmail(), data: {'token': _token});
      return NetworkResponse.fromResponse(response,
          value: response.statusCode == 200);
    } on DioError catch (e) {
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      appDebugPrint('Error: $e');
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> checkExistEmailFromPro(params) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClients(requiredToken: false)
          .get(AppEndpoint.checkExistEmailFromPro(), queryParameters: params);
      return NetworkResponse.fromResponse(response,
          value: response.statusCode == 200);
    } on DioError catch (e) {
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      appDebugPrint('Error: $e');
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> checkNewProEmail(params) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClients(requiredToken: false)
          .post(AppEndpoint.checkNewProEmail(), data: params);
      return NetworkResponse.fromResponse(response,
          value: CheckNewEmailProResult(status: response.statusCode == 200));
    } on DioError catch (e) {
      if (e.response != null &&
          e.response!.statusCode == 400 &&
          e.response!.data != null) {
        return NetworkResponse.fromResponse(e.response!,
            value: CheckNewEmailProResult(
              status: false,
              msg: e.response!.data['error'],
              thumbnail: e.response!.data['thumbnail'],
              roleId: e.response!.data['roleId'],
              displayName: e.response!.data['displayName'],
            ));
      }
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      appDebugPrint('Error: $e');
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> checkEmailPro(params) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClients(requiredToken: false)
          .post(AppEndpoint.checkEmailPro(), data: params);
      return NetworkResponse.fromResponse(response,
          value: CheckNewEmailProResult(status: response.statusCode == 200));
    } on DioError catch (e) {
      if (e.response != null &&
          e.response!.statusCode == 400 &&
          e.response!.data != null) {
        return NetworkResponse.fromResponse(e.response!,
            value: CheckNewEmailProResult(
              status: false,
              msg: e.response!.data['error'],
              thumbnail: e.response!.data['thumbnail'],
              roleId: e.response!.data['roleId'],
              displayName: e.response!.data['displayName'],
            ));
      }
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      appDebugPrint('Error: $e');
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> confirmNewProCode(params) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClients(requiredToken: false)
          .post(AppEndpoint.confirmNewProCode(), data: params);
      return NetworkResponse.fromResponse(response,
          converter: (json) => ConfirmOtp.fromJson(json));
    } on DioError catch (e) {
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      appDebugPrint('Error: $e');
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> confirmResetProPasswordCode(params) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClients(requiredToken: false)
          .post(AppEndpoint.confirmResetProPasswordCode(), data: params);
      return NetworkResponse.fromResponse(response,
          converter: (json) => ConfirmOtp.fromJson(json));
    } on DioError catch (e) {
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      appDebugPrint('Error: $e');
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> createNewPro(params) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClients(requiredToken: false)
          .post(AppEndpoint.createNewPro(), data: params);
      return NetworkResponse.fromResponse(response,
          converter: (json) => Token.fromJson(json), prefix: '');
    } on DioError catch (e) {
      appDebugPrint('DioError: $e');
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      appDebugPrint('Error: $e');
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> getCategoriesList() async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClients(requiredToken: false).get(
          AppEndpoint.getCategoriesList(),
          queryParameters: {'catLanguageCode': null});
      return NetworkResponse.fromResponse(
        response,
        converter: (json) =>
            (json['maincats'] as List).map((e) => MainCat.fromJson(e)).toList(),
      );
    } on DioError catch (e) {
      appDebugPrint('DioError: $e');
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      appDebugPrint('Error: $e');
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> loginPro(params) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClients(requiredToken: false)
          .post(AppEndpoint.loginPro(), data: params);
      return NetworkResponse.fromResponse(response,
          converter: (json) => Token.fromJson(json), prefix: '');
    } on DioError catch (e) {
      appDebugPrint('DioError: $e');
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      appDebugPrint('Error: $e');
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> loginProByPhoneAndFirebase(params) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClients(requiredToken: false)
          .post(AppEndpoint.loginProByPhoneAndFirebase(), data: params);
      return NetworkResponse.fromResponse(response,
          converter: (json) => Token.fromJson(json), prefix: '');
    } on DioError catch (e) {
      appDebugPrint('DioError: $e');
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      appDebugPrint('Error: $e');
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> loginProCrmByQrCode(params) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClients(requiredToken: false)
          .post(AppEndpoint.loginProCrmByQrCode(), data: params);
      return NetworkResponse.fromResponse(response,
          converter: (json) => Token.fromJson(json), prefix: '');
    } on DioError catch (e) {
      appDebugPrint('DioError: $e');
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      appDebugPrint('Error: $e');
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> resetProPassword(params) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClients(requiredToken: false)
          .post(AppEndpoint.resetProPassword(), data: params);
      return NetworkResponse.fromResponse(response,
          value: response.statusCode == 200);
    } on DioError catch (e) {
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      appDebugPrint('Error: $e');
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> updateNewProPasswordByToken(params) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClients(requiredToken: false)
          .post(AppEndpoint.updateNewProPasswordByToken(), data: params);
      return NetworkResponse.fromResponse(response,
          value: response.statusCode == 200);
    } on DioError catch (e) {
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      appDebugPrint('Error: $e');
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> getProInfoByUniqueId(uniqueId) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClients(requiredToken: false).get(
          AppEndpoint.getProInfoByUniqueId(),
          queryParameters: {'uniqueId': uniqueId});
      return NetworkResponse.fromResponse(response,
          converter: (json) => ProsModel.fromJson(json));
    } on DioError catch (e) {
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      appDebugPrint('Error: $e');
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> checkLoginProByEmailOTP(params) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClients(requiredToken: false)
          .post(AppEndpoint.checkLoginProByEmailOTP(), data: params);
      return NetworkResponse.fromResponse(response,
          value: response.statusCode == 200);
    } on DioError catch (e) {
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      appDebugPrint('Error: $e');
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> loginProByEmailOTPConfirm(params) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClients(requiredToken: false)
          .post(AppEndpoint.loginProByEmailOTPConfirm(), data: params);
      return NetworkResponse.fromResponse(response,
          converter: (json) => Token.fromJson(json), prefix: '');
    } on DioError catch (e) {
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      appDebugPrint('Error: $e');
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> getAssetsForProFromUniversalDashboard(
      proUniqueId) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClients(requiredToken: false).get(
          AppEndpoint.getAssetsForProFromUniversalDashboard(),
          queryParameters: {'proUniqueId': proUniqueId});
      return NetworkResponse.fromResponse(response,
          converter: (json) => OrgProInfo.fromJson(json));
    } on DioError catch (e) {
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      appDebugPrint('Error: $e');
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse<List<Country>>> getPhoneAvailable() async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClients(requiredToken: false).get(
          AppEndpoint.getPhoneAvailable(),
          queryParameters: {'phoneAvailable': 1});
      return NetworkResponse.fromResponse(response,
          converter: (json) => (json as List<dynamic>)
              .map<Country>((i) => Country.fromJson(i))
              .toList());
    } on DioError catch (e) {
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      appDebugPrint('Error: $e');
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse<List<DateFormatModel>>> getDateFormatList() async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClients(requiredToken: false)
          .get(AppEndpoint.getDateFormatList());
      return NetworkResponse.fromResponse(response,
          converter: (json) => (json as List<dynamic>)
              .map<DateFormatModel>((i) => DateFormatModel.fromJson(i))
              .toList());
    } on DioError catch (e) {
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      appDebugPrint('Error: $e');
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse<List<TimeFormatModel>>> getTimeFormatList() async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClients(requiredToken: false)
          .get(AppEndpoint.getTimeFormatList());
      return NetworkResponse.fromResponse(response,
          converter: (json) => (json as List<dynamic>)
              .map<TimeFormatModel>((i) => TimeFormatModel.fromJson(i))
              .toList());
    } on DioError catch (e) {
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      appDebugPrint('Error: $e');
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse<List<Country>>> getCountryAvailable() async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClients(requiredToken: false).get(
          AppEndpoint.getCountryAvailable(),
          queryParameters: {'available': 1});
      return NetworkResponse.fromResponse(response,
          converter: (json) => (json as List<dynamic>)
              .map<Country>((i) => Country.fromJson(i))
              .toList());
    } on DioError catch (e) {
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      appDebugPrint('Error: $e');
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse<List<IndustryModel>>> getIndustriesList() async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClients(requiredToken: false).get(
          AppEndpoint.getIndustriesList(),
          queryParameters: {'languageCode': appPrefs.languageCode});
      return NetworkResponse.fromResponse(response,
          converter: (json) => (json as List<dynamic>)
              .map<IndustryModel>((i) => IndustryModel.fromJson(i))
              .toList());
    } on DioError catch (e) {
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      appDebugPrint('Error: $e');
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> getLanguageAvailable() async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClients(requiredToken: false).get(
          AppEndpoint.getLanguageAvailable(),
          queryParameters: {'available': 1});
      return NetworkResponse.fromResponse(response,
          converter: (json) => (json as List<dynamic>)
              .map<LanguageModel>((i) => LanguageModel.fromJson(i))
              .toList());
    } on DioError catch (e) {
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      appDebugPrint('Error: $e');
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> getLanguageForSpeakAvailable() async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClients(requiredToken: false).get(
          AppEndpoint.getLanguageForSpeakAvailable(),
          queryParameters: {'availableForSpeak': 1});
      return NetworkResponse.fromResponse(response,
          converter: (json) => (json as List<dynamic>)
              .map<LanguageModel>((i) => LanguageModel.fromJson(i))
              .toList());
    } on DioError catch (e) {
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      appDebugPrint('Error: $e');
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> getTerminalsFromPro(terminalType) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClients(
              requiredToken: true, token: await appPrefs.getNormalToken())
          .get(AppEndpoint.getTerminalsFromPro(),
              queryParameters: {'terminalType': terminalType});
      return NetworkResponse.fromResponse(response,
          converter: (json) =>
              (json as List).map((e) => Terminal.fromJson(e)).toList());
    } on DioError catch (e) {
      appDebugPrint('DioError: $e');
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      appDebugPrint('Error: $e');
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> getVideoConfigFromPlayerVimeo(vimeoId) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClients(requiredToken: false).get(
          AppEndpoint.getVideoConfigFromPlayerVimeo(),
          queryParameters: {'videoId': vimeoId});
      return NetworkResponse.fromResponse(response,
          converter: (json) => VimeoConfig.fromJson(json));
    } on DioError catch (e) {
      appDebugPrint('DioError: $e');
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      appDebugPrint('Error: $e');
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> updateTerminalFromPro(params) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClients(
              requiredToken: true, token: await appPrefs.getNormalToken())
          .put(AppEndpoint.updateTerminalFromPro(), data: params);
      return NetworkResponse.fromResponse(response,
          value: response.statusCode == 200);
    } on DioError catch (e) {
      appDebugPrint('DioError: $e');
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      appDebugPrint('Error: $e');
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> sendPropertyToTerminal(params) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClients(
              requiredToken: true, token: await appPrefs.getNormalToken())
          .post(AppEndpoint.sendPropertyToTerminal(), data: params);
      return NetworkResponse.fromResponse(response,
          value: response.statusCode == 200);
    } on DioError catch (e) {
      appDebugPrint('DioError: $e');
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      appDebugPrint('Error: $e');
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> removePropertyFromTerminal(params) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClients(
              requiredToken: true, token: await appPrefs.getNormalToken())
          .delete(AppEndpoint.removePropertyFromTerminal(), data: params);
      return NetworkResponse.fromResponse(response,
          value: response.statusCode == 200);
    } on DioError catch (e) {
      appDebugPrint('DioError: $e');
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      appDebugPrint('Error: $e');
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> updateProduct(params) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClients(
              requiredToken: true, token: await appPrefs.getNormalToken())
          .put(AppEndpoint.updateProduct(), data: params);
      return NetworkResponse.fromResponse(response,
          value: response.statusCode == 200);
    } on DioError catch (e) {
      appDebugPrint('DioError: $e');
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      appDebugPrint('Error: $e');
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> takeMissionFromSeller(missionId) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClients(
              requiredToken: true, token: await appPrefs.getNormalToken())
          .post(AppEndpoint.takeMissionFromSeller(),
              data: {'missionId': missionId});
      return NetworkResponse.fromResponse(response,
          value: response.statusCode == 200);
    } on DioError catch (e) {
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> googlePlaceAutoComplete(
      String keyword, String countryCode,
      [String? types]) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClients(requiredToken: true)
          .get(AppEndpoint.googlePlaceAutoComplete(), queryParameters: {
        'value': keyword,
        'country': countryCode,
        'types': types
      });
      if (response.data['status'] == 'OK') {
        return NetworkResponse.fromResponse(response,
            prefix: 'predictions',
            converter: (json) => (json as List<dynamic>)
                .map<AutocompletePrediction>(
                    (i) => AutocompletePrediction.fromJson(i))
                .toList());
      } else {
        return NetworkResponse();
      }
    } on DioError catch (e) {
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      appDebugPrint('Error: $e');
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> googlePlaceSeachPlaceByText(String keyword,
      [String? types]) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClients(requiredToken: false).get(
          'https://maps.googleapis.com/maps/api/place/textsearch/json',
          queryParameters: {
            'query': keyword,
            'key': googleApiKey,
            'type': types
          });
      return NetworkResponse.fromResponse(response,
          prefix: 'results',
          converter: (json) => (json as List<dynamic>)
              .map<SeachPlaceResultModel>(
                  (i) => SeachPlaceResultModel.fromJson(i))
              .toList());
    } on DioError catch (e) {
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      appDebugPrint('Error: $e');
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> googlePlaceSeachPlaceDetailById(placeId) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClients(requiredToken: false).get(
          'https://maps.googleapis.com/maps/api/place/details/json',
          queryParameters: {'place_id': placeId, 'key': googleApiKey});
      return NetworkResponse.fromResponse(response,
          prefix: 'result',
          converter: (json) => SeachPlaceResultDetailModel.fromJson(json));
    } on DioError catch (e) {
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      appDebugPrint('Error: $e');
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> googleGeocode(latlng) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClients(requiredToken: false).get(
          'https://maps.googleapis.com/maps/api/geocode/json',
          queryParameters: {'latlng': latlng, 'key': googleApiKey});
      return NetworkResponse.fromResponse(response,
          prefix: 'results',
          converter: (json) => (json as List<dynamic>)
              .map<SeachPlaceResultDetailModel>(
                  (i) => SeachPlaceResultDetailModel.fromJson(i))
              .toList());
    } on DioError catch (e) {
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      appDebugPrint('Error: $e');
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> googlePlaceSearchById(String placeId) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClients(requiredToken: true).get(
          AppEndpoint.googlePlaceSearchById(),
          queryParameters: {'placeId': placeId});
      return NetworkResponse.fromResponse(response,
          prefix: '', converter: (json) => DetailsResponse.fromJson(json));
    } on DioError catch (e) {
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      appDebugPrint('Error: $e');
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> getProductCategories() async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClients(requiredToken: true).get(
          AppEndpoint.getProductCategories(),
          queryParameters: {'display': 1});
      return NetworkResponse.fromResponse(response,
          converter: (json) => (json as List<dynamic>)
              .map<ProductCategory>((i) => ProductCategory.fromJson(i))
              .toList());
    } on DioError catch (e) {
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      appDebugPrint('Error: $e');
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> getProductsFromPro(params) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClients(requiredToken: true)
          .get(AppEndpoint.getProductsFromPro(), queryParameters: params);
      return NetworkResponse.fromResponse(response,
          converter: (json) => (json as List<dynamic>)
              .map<Product>((i) => Product.fromJson(i))
              .toList());
    } on DioError catch (e) {
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      appDebugPrint('Error: $e');
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> getStatusList() async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClients(requiredToken: true)
          .get(AppEndpoint.getStatusList());
      return NetworkResponse.fromResponse(response,
          converter: (json) => (json as List<dynamic>)
              .map<ProductStatus>((i) => ProductStatus.fromJson(i))
              .toList());
    } on DioError catch (e) {
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      appDebugPrint('Error: $e');
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> getAllPublicEventsFromSearch(params) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      final token = await appPrefs.getNormalToken();
      Response response = await AppClients(requiredToken: true, token: token)
          .get(AppEndpoint.getAllPublicEventsFromSearch(),
              queryParameters: params);
      return NetworkResponse.fromResponse(response,
          converter: (json) => (json as List<dynamic>)
              .map<PublishEvent>((i) => PublishEvent.fromJson(i))
              .toList());
    } on DioError catch (e) {
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      appDebugPrint('Error: $e');
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> getAllPublicEventsAroundMe(params) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      final token = await appPrefs.getNormalToken();
      Response response = await AppClients(requiredToken: true, token: token)
          .get(AppEndpoint.getAllPublicEventsAroundMe(),
              queryParameters: params);
      return NetworkResponse.fromResponse(response,
          converter: (json) => (json as List<dynamic>)
              .map<PublishEvent>((i) => PublishEvent.fromJson(i))
              .toList());
    } on DioError catch (e) {
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      appDebugPrint('Error: $e');
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> getPublicEventsListFromUser(params) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      final token = await appPrefs.getNormalToken();
      Response response = await AppClients(requiredToken: true, token: token)
          .get(AppEndpoint.getPublicEventsListFromUser(),
              queryParameters: params);
      return NetworkResponse.fromResponse(response,
          converter: (json) => (json as List<dynamic>)
              .map<PublishEvent>((i) => PublishEvent.fromJson(i))
              .toList());
    } on DioError catch (e) {
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      appDebugPrint('Error: $e');
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> getSuggestionEventsFromUser(params) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      final token = await appPrefs.getNormalToken();
      Response response = await AppClients(requiredToken: true, token: token)
          .get(AppEndpoint.getSuggestionEventsFromUser(),
              queryParameters: params);
      return NetworkResponse.fromResponse(response,
          converter: (json) => (json as List<dynamic>)
              .map<PublishEvent>((i) => PublishEvent.fromJson(i))
              .toList());
    } on DioError catch (e) {
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      appDebugPrint('Error: $e');
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> getPublicEventsCategories(languageCode) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      final token = await appPrefs.getNormalToken();
      Response response = await AppClients(requiredToken: true, token: token)
          .get(AppEndpoint.getPublicEventsCategories(),
              queryParameters: {'languageCode': languageCode});
      return NetworkResponse.fromResponse(response,
          converter: (json) => (json as List<dynamic>)
              .map<PublishEventCategory>(
                  (i) => PublishEventCategory.fromJson(i))
              .toList());
    } on DioError catch (e) {
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      appDebugPrint('Error: $e');
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> getFavoritePublicEventsCategoriesFromUser(
      languageCode) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      final token = await appPrefs.getNormalToken();
      Response response = await AppClients(requiredToken: true, token: token)
          .get(AppEndpoint.getFavoritePublicEventsCategoriesFromUser(),
              queryParameters: {'languageCode': languageCode});
      return NetworkResponse.fromResponse(response,
          converter: (json) => (json as List<dynamic>)
              .map<PublishEventCategory>(
                  (i) => PublishEventCategory.fromJson(i))
              .toList());
    } on DioError catch (e) {
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      appDebugPrint('Error: $e');
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> getLikedPublicEventsFromUser() async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      final token = await appPrefs.getNormalToken();
      Response response = await AppClients(requiredToken: true, token: token)
          .get(AppEndpoint.getLikedPublicEventsFromUser());
      return NetworkResponse.fromResponse(response,
          converter: (json) => (json as List<dynamic>)
              .map<PublishEvent>((i) => PublishEvent.fromJson(i))
              .toList());
    } on DioError catch (e) {
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      appDebugPrint('Error: $e');
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> getTicketListFromPublicEvent(publicEventId) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      final token = await appPrefs.getNormalToken();
      Response response = await AppClients(requiredToken: true, token: token)
          .get(AppEndpoint.getTicketListFromPublicEvent(),
              queryParameters: {'publicEventId': publicEventId});
      return NetworkResponse.fromResponse(response,
          converter: (json) => (json as List<dynamic>)
              .map<TicketInfoModel>((i) => TicketInfoModel.fromJson(i))
              .toList());
    } on DioError catch (e) {
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      appDebugPrint('Error: $e');
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> addUserLikeToPublicEvent(publicEventId) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      final token = await appPrefs.getNormalToken();
      Response response = await AppClients(requiredToken: true, token: token)
          .post(AppEndpoint.addUserLikeToPublicEvent(),
              data: {'publicEventId': publicEventId});
      return NetworkResponse.fromResponse(response,
          value: response.statusCode == 200);
    } on DioError catch (e) {
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      appDebugPrint('Error: $e');
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> setFavoriteCatForUser(catId) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      final token = await appPrefs.getNormalToken();
      Response response = await AppClients(requiredToken: true, token: token)
          .post(AppEndpoint.setFavoriteCatForUser(), data: {'catId': catId});
      return NetworkResponse.fromResponse(response,
          value: response.statusCode == 200);
    } on DioError catch (e) {
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      appDebugPrint('Error: $e');
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> removeFavoriteCatForUser(catId) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      final token = await appPrefs.getNormalToken();
      Response response = await AppClients(requiredToken: true, token: token)
          .delete(AppEndpoint.removeFavoriteCatForUser(),
              data: {'catId': catId});
      return NetworkResponse.fromResponse(response,
          value: response.statusCode == 200);
    } on DioError catch (e) {
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      appDebugPrint('Error: $e');
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> addSeenSuggestionEventFromUser(
      publicEventId, utcModifier) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      final token = await appPrefs.getNormalToken();
      Response response = await AppClients(requiredToken: true, token: token)
          .post(AppEndpoint.addSeenSuggestionEventFromUser(), data: {
        'publicEventId': publicEventId,
        'utcModifier': utcModifier
      });
      return NetworkResponse.fromResponse(response,
          value: response.statusCode == 200);
    } on DioError catch (e) {
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      appDebugPrint('Error: $e');
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> bookFreeTicketForPublicEvents(oowooTicketId) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      final token = await appPrefs.getNormalToken();
      Response response = await AppClients(requiredToken: true, token: token)
          .post(AppEndpoint.bookFreeTicketForPublicEvents(),
              data: {'oowooTicketId': oowooTicketId});
      return NetworkResponse.fromResponse(response,
          value: response.statusCode == 200);
    } on DioError catch (e) {
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      appDebugPrint('Error: $e');
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> getAllowedFeaturesForOowooApp() async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      final token = await appPrefs.getNormalToken();
      Response response = await AppClients(requiredToken: true, token: token)
          .get(AppEndpoint.getAllowedFeaturesForOowooApp());
      return NetworkResponse.fromResponse(response,
          converter: (json) => (json as List<dynamic>)
              .map<AllowedFeatureModel>((i) => AllowedFeatureModel.fromJson(i))
              .toList());
    } on DioError catch (e) {
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      appDebugPrint('Error: $e');
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> getDashboardInfos() async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      final token = await appPrefs.getNormalToken();
      Response response = await AppClients(requiredToken: true, token: token)
          .get(AppEndpoint.getDashboardInfos());
      return NetworkResponse.fromResponse(response,
          converter: (json) => (json as List<dynamic>)
              .map<DashboardInfo>((i) => DashboardInfo.fromJson(i))
              .toList());
    } on DioError catch (e) {
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      appDebugPrint('Error: $e');
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> createCustomerCardStripe(params) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      final token = await appPrefs.getNormalToken();
      Response response = await AppClients(requiredToken: true, token: token)
          .post(AppEndpoint.createCustomerCardStripe(), data: params);
      return NetworkResponse.fromResponse(response,
          converter: (json) => CardStripeModel.fromJson(json));
    } on DioError catch (e) {
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      appDebugPrint('Error: $e');
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> deleteCustomerCardStripe(cardId) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      final token = await appPrefs.getNormalToken();
      Response response = await AppClients(requiredToken: true, token: token)
          .delete(AppEndpoint.deleteCustomerCardStripe(),
              data: {'cardId': cardId});
      return NetworkResponse.fromResponse(response,
          value: response.statusCode == 200);
    } on DioError catch (e) {
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      appDebugPrint('Error: $e');
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> getCustomerCardsStripe(page, pageSize) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      final token = await appPrefs.getNormalToken();
      Response response = await AppClients(requiredToken: true, token: token)
          .get(AppEndpoint.getCustomerCardsStripe(),
              queryParameters: {'page': page, 'page_size': pageSize});
      return NetworkResponse.fromResponse(response,
          converter: (json) => (json as List<dynamic>)
              .map<CardStripeModel>((i) => CardStripeModel.fromJson(i))
              .toList());
    } on DioError catch (e) {
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> setCustomerCardDefaultStripe(cardId) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      final token = await appPrefs.getNormalToken();
      Response response = await AppClients(requiredToken: true, token: token)
          .put(AppEndpoint.setCustomerCardDefaultStripe(),
              data: {'cardId': cardId});
      return NetworkResponse.fromResponse(response,
          value: response.statusCode == 200);
    } on DioError catch (e) {
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      appDebugPrint('Error: $e');
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> becomeSeller(confirmed) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      final token = await appPrefs.getNormalToken();
      Response response = await AppClients(requiredToken: true, token: token)
          .post(AppEndpoint.becomeSeller(), data: {'confirmed': confirmed});
      return NetworkResponse.fromResponse(response,
          value: response.statusCode == 200);
    } on DioError catch (e) {
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      appDebugPrint('Error: $e');
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> updateImageForBecomeSeller(fileType, path) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      String fileName = path.split('/').last;
      FormData formData = FormData.fromMap({
        "fileType": fileType,
        "fileName": MultipartFile.fromBytes(File(path).readAsBytesSync(),
            filename: fileName),
      });
      final token = await appPrefs.getNormalToken();
      Response response = await AppClients(requiredToken: true, token: token)
          .post(AppEndpoint.updateImageForBecomeSeller(), data: formData);
      return NetworkResponse.fromResponse(response,
          value: response.data['values']['fileName']);
    } on DioError catch (e) {
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      appDebugPrint('Error: $e');
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> quitSellerProgram(temporary) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      final token = await appPrefs.getNormalToken();
      Response response = await AppClients(requiredToken: true, token: token)
          .post(AppEndpoint.quitSellerProgram(),
              data: {'temporary': temporary});
      return NetworkResponse.fromResponse(response,
          value: response.statusCode == 200);
    } on DioError catch (e) {
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      appDebugPrint('Error: $e');
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> recoverySellerProgram() async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      final token = await appPrefs.getNormalToken();
      Response response = await AppClients(requiredToken: true, token: token)
          .post(AppEndpoint.recoverySellerProgram());
      return NetworkResponse.fromResponse(response,
          value: response.statusCode == 200);
    } on DioError catch (e) {
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      appDebugPrint('Error: $e');
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> getTncOowooApp() async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClients(requiredToken: false).get(
          AppEndpoint.getTncOowooApp(),
          queryParameters: {'languageCode': appPrefs.languageCode});
      return NetworkResponse.fromResponse(response,
          converter: (json) => json['text']);
    } on DioError catch (e) {
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      appDebugPrint('Error: $e');
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> getTncOowooSeller() async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClients(requiredToken: false).get(
          AppEndpoint.getTncOowooSeller(),
          queryParameters: {'languageCode': appPrefs.languageCode});
      return NetworkResponse.fromResponse(response,
          converter: (json) => json['text']);
    } on DioError catch (e) {
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      appDebugPrint('Error: $e');
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> birthsOnThisDay(date) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClients(requiredToken: false)
          .get('https://byabbe.se/on-this-day/$date/births.json');
      return NetworkResponse.fromResponse(response,
          prefix: 'births',
          converter: (json) => (json as List<dynamic>)
              .map<OnThisDayModel>((i) => OnThisDayModel.fromJson(i))
              .toList());
    } on DioError catch (e) {
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> deathsOnThisDay(date) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClients(requiredToken: false)
          .get('https://byabbe.se/on-this-day/$date/deaths.json');
      return NetworkResponse.fromResponse(response,
          prefix: 'deaths',
          converter: (json) => (json as List<dynamic>)
              .map<OnThisDayModel>((i) => OnThisDayModel.fromJson(i))
              .toList());
    } on DioError catch (e) {
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> eventsOnThisDay(date) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClients(requiredToken: false)
          .get('https://byabbe.se/on-this-day/$date/events.json');
      return NetworkResponse.fromResponse(response,
          prefix: 'events',
          converter: (json) => (json as List<dynamic>)
              .map<OnThisDayModel>((i) => OnThisDayModel.fromJson(i))
              .toList());
    } on DioError catch (e) {
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> pexelsSearch(params) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClients(
              requiredToken: true,
              isAuthorizationCustom: true,
              token: PEXEL_API_KEY)
          .get('https://api.pexels.com/v1/search', queryParameters: params);
      return NetworkResponse.fromResponse(response,
          prefix: 'photos',
          converter: (json) => (json as List<dynamic>)
              .map<PexelPhotoModel>((i) => PexelPhotoModel.fromJson(i))
              .toList());
    } on DioError catch (e) {
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> weatherForecast(params) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClients(
        requiredToken: false,
        enableErrorHandler: false,
      ).get('https://api.weatherapi.com/v1/forecast.json',
          queryParameters: params);
      return NetworkResponse.fromResponse(response,
          prefix: 'forecast',
          converter: (json) => WeatherForecast.fromJson(json));
    } on DioError catch (e) {
      appDebugPrint('DioError: $e');
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      appDebugPrint('catch: $e');
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> weatherHistory(params) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClients(
        requiredToken: false,
        enableErrorHandler: false,
      ).get('https://api.weatherapi.com/v1/history.json',
          queryParameters: params);
      return NetworkResponse.fromResponse(response,
          prefix: 'forecast',
          converter: (json) => WeatherForecast.fromJson(json));
    } on DioError catch (e) {
      appDebugPrint('DioError: $e');
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      appDebugPrint('catch: $e');
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> getMissionsFromSeller(params) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClients(
              requiredToken: true, token: await appPrefs.getNormalToken())
          .get(AppEndpoint.getMissionsFromSeller(), queryParameters: params);
      return NetworkResponse.fromResponse(response,
          converter: (json) => (json as List<dynamic>)
              .map<SellerMissionModel>((i) => SellerMissionModel.fromJson(i))
              .toList());
    } on DioError catch (e) {
      appDebugPrint('DioError: $e');
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      appDebugPrint('catch: $e');
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> getMissionTypes(params) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClients(
              requiredToken: true, token: await appPrefs.getNormalToken())
          .get(AppEndpoint.getMissionTypes(), queryParameters: params);
      return NetworkResponse.fromResponse(response,
          converter: (json) => (json as List<dynamic>)
              .map<MissionTypeModel>((i) => MissionTypeModel.fromJson(i))
              .toList());
    } on DioError catch (e) {
      appDebugPrint('DioError: $e');
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      appDebugPrint('catch: $e');
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> getOoBRPlaceInfoById(params) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClients(
              requiredToken: true, token: await appPrefs.getNormalToken())
          .get(AppEndpoint.getOoBRPlaceInfoById(), queryParameters: params);
      return NetworkResponse.fromResponse(response,
          converter: (json) => BrPlaceModel.fromJson(json));
    } on DioError catch (e) {
      appDebugPrint('DioError: $e');
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      appDebugPrint('catch: $e');
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> getOoBRPlacesAroundMe(params) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClients(
              requiredToken: true, token: await appPrefs.getNormalToken())
          .get(AppEndpoint.getOoBRPlacesAroundMe(), queryParameters: params);
      return NetworkResponse.fromResponse(response,
          converter: (json) => (json as List<dynamic>)
              .map<BrPlaceModel>((i) => BrPlaceModel.fromJson(i))
              .toList());
    } on DioError catch (e) {
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      appDebugPrint('Error: $e');
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> getOoBRPlacesByFilterValues(params) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClients(
              requiredToken: true, token: await appPrefs.getNormalToken())
          .get(AppEndpoint.getOoBRPlacesByFilterValues(),
              queryParameters: params);
      return NetworkResponse.fromResponse(response,
          converter: (json) => (json as List<dynamic>)
              .map<BrPlaceModel>((i) => BrPlaceModel.fromJson(i))
              .toList());
    } on DioError catch (e) {
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      appDebugPrint('Error: $e');
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> getFavoriteOobrPlaces() async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClients(
              requiredToken: true, token: await appPrefs.getNormalToken())
          .get(AppEndpoint.getFavoriteOobrPlaces());
      return NetworkResponse.fromResponse(response,
          converter: (json) => (json as List<dynamic>)
              .map<BrPlaceModel>((i) => BrPlaceModel.fromJson(i))
              .toList());
    } on DioError catch (e) {
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      appDebugPrint('Error: $e');
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> updateFilterValueForOobr(params) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClients(
              requiredToken: true, token: await appPrefs.getNormalToken())
          .post(AppEndpoint.updateFilterValueForOobr(), data: params);
      return NetworkResponse.fromResponse(response,
          converter: (json) => json['updateFilterValueForOobr']);
    } on DioError catch (e) {
      if (e.response != null &&
          e.response!.statusCode == 400 &&
          e.response!.data != null) {
        return NetworkResponse.fromResponse(e.response!,
            value: e.response!.data['error']);
      }
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      appDebugPrint('catch: $e');
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> getPromotedVideosForBar(oobrPlaceId) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClients(
              requiredToken: true, token: await appPrefs.getNormalToken())
          .get(AppEndpoint.getPromotedVideosForBar(),
              queryParameters: {'oobrPlaceId': oobrPlaceId});
      return NetworkResponse.fromResponse(response,
          converter: (json) =>
              (json as List<dynamic>).map<String>((i) => i).toList());
    } on DioError catch (e) {
      appDebugPrint('DioError: $e');
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      appDebugPrint('catch: $e');
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> getPromotedVideosForOobrPlaces(lat, lng) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClients(
              requiredToken: true, token: await appPrefs.getNormalToken())
          .get(AppEndpoint.getPromotedVideosForOobrPlaces(),
              queryParameters: {'lat': lat, 'lng': lng});
      return NetworkResponse.fromResponse(response,
          converter: (json) => (json as List<dynamic>)
              .map<BrPromotedVideoModel>(
                  (i) => BrPromotedVideoModel.fromJson(i))
              .toList());
    } on DioError catch (e) {
      appDebugPrint('DioError: $e');
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      appDebugPrint('catch: $e');
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> getPromotedVideosForOoh(lat, lng) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClients(
              requiredToken: true, token: await appPrefs.getNormalToken())
          .get(AppEndpoint.getPromotedVideosForOoh(),
              queryParameters: {'lat': lat, 'lng': lng});
      return NetworkResponse.fromResponse(response,
          converter: (json) => (json as List<dynamic>)
              .map<BrPromotedVideoModel>(
                  (i) => BrPromotedVideoModel.fromJson(i))
              .toList());
    } on DioError catch (e) {
      appDebugPrint('DioError: $e');
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      appDebugPrint('catch: $e');
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> getOoBRCategories(languageCode) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClients(
              requiredToken: true, token: await appPrefs.getNormalToken())
          .get(AppEndpoint.getOoBRCategories(),
              queryParameters: {'languageCode': languageCode});
      return NetworkResponse.fromResponse(response,
          converter: (json) => (json as List<dynamic>)
              .map<BrPlaceCateModel>((i) => BrPlaceCateModel.fromJson(i))
              .toList());
    } on DioError catch (e) {
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      appDebugPrint('Error: $e');
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> getOoBRFilters(languageCode) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClients(
              requiredToken: true, token: await appPrefs.getNormalToken())
          .get(AppEndpoint.getOoBRFilters(),
              queryParameters: {'languageCode': languageCode});
      return NetworkResponse.fromResponse(response,
          converter: (json) => (json as List<dynamic>)
              .map<BrPlaceFilterModel>((i) => BrPlaceFilterModel.fromJson(i))
              .toList());
    } on DioError catch (e) {
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      appDebugPrint('Error: $e');
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> checkBarGooglePlaceIdByContributor(
      googlePlaceId) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClients(
              requiredToken: true, token: await appPrefs.getNormalToken())
          .get(AppEndpoint.checkBarGooglePlaceIdByContributor(),
              queryParameters: {'googlePlaceId': googlePlaceId});
      return NetworkResponse.fromResponse(response, value: '');
    } on DioError catch (e) {
      if (e.response != null &&
          e.response!.statusCode == 400 &&
          e.response!.data != null) {
        return NetworkResponse.fromResponse(e.response!,
            value: e.response!.data['oobrPlaceId']);
      }
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      appDebugPrint('Error: $e');
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> createNewOobrPlaceByContributor(params) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClients(
              requiredToken: true, token: await appPrefs.getNormalToken())
          .post(AppEndpoint.createNewOobrPlaceByContributor(), data: params);
      return NetworkResponse.fromResponse(response,
          value: response.statusCode == 200);
    } on DioError catch (e) {
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      appDebugPrint('Error: $e');
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> addOobrPlaceToFavorites(oobrPlaceId) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClients(
              requiredToken: true, token: await appPrefs.getNormalToken())
          .post(AppEndpoint.addOobrPlaceToFavorites(),
              data: {'oobrPlaceId': oobrPlaceId});
      return NetworkResponse.fromResponse(response,
          value: response.statusCode == 200);
    } on DioError catch (e) {
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      appDebugPrint('Error: $e');
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> getOobrPlaceSchedules(oobrPlaceId) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClients(
              requiredToken: true, token: await appPrefs.getNormalToken())
          .get(AppEndpoint.getOobrPlaceSchedules(),
              queryParameters: {'oobrPlaceId': oobrPlaceId});
      return NetworkResponse.fromResponse(response,
          converter: (json) => (json as List<dynamic>)
              .map<BrPlaceScheduleModel>(
                  (i) => BrPlaceScheduleModel.fromJson(i))
              .toList());
    } on DioError catch (e) {
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      appDebugPrint('Error: $e');
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> getOobrPlaceMenuList(
      oobrPlaceId, languageCode) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClients(
              requiredToken: true, token: await appPrefs.getNormalToken())
          .get(AppEndpoint.getOobrPlaceMenuList(), queryParameters: {
        'oobrPlaceId': oobrPlaceId,
        'languageCode': languageCode
      });
      return NetworkResponse.fromResponse(response,
          converter: (json) => (json as List<dynamic>)
              .map<BrPlaceMenu>((i) => BrPlaceMenu.fromJson(i))
              .toList());
    } on DioError catch (e) {
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      appDebugPrint('Error: $e');
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> getOobrPlaceMenuContent(
      oobrPlaceId, languageCode) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClients(
              requiredToken: true, token: await appPrefs.getNormalToken())
          .get(AppEndpoint.getOobrPlaceMenuContent(), queryParameters: {
        'oobrPlaceId': oobrPlaceId,
        'languageCode': languageCode
      });
      return NetworkResponse.fromResponse(response,
          converter: (json) => BrPlaceMenuContent.fromJson(json));
    } on DioError catch (e) {
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      appDebugPrint('Error: $e');
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> getOptionsForItem(itemId, languageCode) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClients(
              requiredToken: true, token: await appPrefs.getNormalToken())
          .get(AppEndpoint.getOptionsForItem(), queryParameters: {
        'itemId': itemId,
        'languageCode': languageCode
      });
      return NetworkResponse.fromResponse(response,
          converter: (json) => (json as List<dynamic>)
              .map<PlaceMenuItemOption>((i) => PlaceMenuItemOption.fromJson(i))
              .toList());
    } on DioError catch (e) {
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      appDebugPrint('Error: $e');
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> getAllergensForItem(itemId, languageCode) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClients(
              requiredToken: true, token: await appPrefs.getNormalToken())
          .get(AppEndpoint.getAllergensForItem(), queryParameters: {
        'itemId': itemId,
        'languageCode': languageCode
      });
      return NetworkResponse.fromResponse(response,
          converter: (json) => (json as List<dynamic>)
              .map<PlaceMenuAllergens>((i) => PlaceMenuAllergens.fromJson(i))
              .toList());
    } on DioError catch (e) {
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      appDebugPrint('Error: $e');
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> getContactsList(params) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClients(
              requiredToken: true, token: await appPrefs.getNormalToken())
          .get(AppEndpoint.getContactsList(), queryParameters: params);
      return NetworkResponse.fromResponse(response,
          converter: (json) => (json as List<dynamic>)
              .map<UserModel>((i) => UserModel.fromJson(i))
              .toList());
    } on DioError catch (e) {
      appDebugPrint('DioError: $e');
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      appDebugPrint('Error: $e');
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> addUserInContactsList(params) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClients(
              requiredToken: true, token: await appPrefs.getNormalToken())
          .post(AppEndpoint.addUserInContactsList(), data: params);
      return NetworkResponse.fromResponse(response,
          converter: (json) => UserModel.fromJson(json));
    } on DioError catch (e) {
      return NetworkResponse.withErrorRequest(e);
    }
  }

  @override
  Future<NetworkResponse> deleteContact(id) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClients(
              requiredToken: true, token: await appPrefs.getNormalToken())
          .delete(AppEndpoint.deleteContact(), data: {'id': id});
      return NetworkResponse.fromResponse(response,
          value: response.statusCode == 200);
    } on DioError catch (e) {
      return NetworkResponse.withErrorRequest(e);
    }
  }

  @override
  Future<NetworkResponse> updateCommentForContact(params) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClients(
              requiredToken: true, token: await appPrefs.getNormalToken())
          .put(AppEndpoint.updateCommentForContact(), data: params);
      return NetworkResponse.fromResponse(response,
          value: response.statusCode == 200);
    } on DioError catch (e) {
      return NetworkResponse.withErrorRequest(e);
    }
  }

  @override
  Future<NetworkResponse> getExistingUserByPhone(params) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClients(
              requiredToken: true, token: await appPrefs.getNormalToken())
          .get(AppEndpoint.getExistingUserByPhone(), queryParameters: params);
      return NetworkResponse.fromResponse(response,
          converter: (json) => UserModel.fromJson(json));
    } on DioError catch (e) {
      return NetworkResponse.withErrorRequest(e);
    }
  }

  @override
  Future<NetworkResponse> createOrderForOobrPlace(params) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClients(
              requiredToken: true, token: await appPrefs.getNormalToken())
          .post(AppEndpoint.createOrderForOobrPlace(), data: params);
      return NetworkResponse.fromResponse(response,
          converter: (json) => (json as List<dynamic>)
              .map<UserModel>((i) => UserModel.fromJson(i))
              .toList());
    } on DioError catch (e) {
      appDebugPrint('DioError: $e');
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      appDebugPrint('Error: $e');
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> getAnythingFromQr(params) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClients(
              requiredToken: true, token: await appPrefs.getNormalToken())
          .get(AppEndpoint.getAnythingFromQr(), queryParameters: params);
      return NetworkResponse.fromResponse(response,
          converter: (json) => QrDataModel.fromJson(json));
    } on DioError catch (e) {
      appDebugPrint('DioError: $e');
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      appDebugPrint('Error: $e');
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> getDetailOobrInvitations(oobrEventId) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClients(
              requiredToken: true, token: await appPrefs.getNormalToken())
          .get(AppEndpoint.getDetailOobrInvitations(),
              queryParameters: {'oobrEventId': oobrEventId});
      return NetworkResponse.fromResponse(response,
          converter: (json) => BrPlaceInviteModel.fromJson(json));
    } on DioError catch (e) {
      appDebugPrint('DioError: $e');
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      appDebugPrint('Error: $e');
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> inviteContactsToOobrPlace(params) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClients(
              requiredToken: true, token: await appPrefs.getNormalToken())
          .post(AppEndpoint.inviteContactsToOobrPlace(), data: params);
      return NetworkResponse.fromResponse(response,
          value: response.statusCode == 200);
    } on DioError catch (e) {
      appDebugPrint('DioError: $e');
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      appDebugPrint('Error: $e');
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> answerOobrInvitation(params) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClients(
              requiredToken: true, token: await appPrefs.getNormalToken())
          .post(AppEndpoint.answerOobrInvitation(), data: params);
      return NetworkResponse.fromResponse(response,
          value: response.statusCode == 200);
    } on DioError catch (e) {
      appDebugPrint('DioError: $e');
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      appDebugPrint('Error: $e');
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> getEmergencyInfos(params) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClients(
              requiredToken: true, token: await appPrefs.getNormalToken())
          .get(AppEndpoint.getEmergencyInfos(), queryParameters: params);
      return NetworkResponse.fromResponse(response,
          converter: (json) => EmergencyResult.fromJson(json));
    } on DioError catch (e) {
      appDebugPrint('DioError: $e');
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      appDebugPrint('Error: $e');
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> sendEmergencyRequest(params) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClients(
              requiredToken: true, token: await appPrefs.getNormalToken())
          .post(AppEndpoint.sendEmergencyRequest(), data: params);
      return NetworkResponse.fromResponse(response,
          value: response.statusCode == 200);
    } on DioError catch (e) {
      appDebugPrint('DioError: $e');
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      appDebugPrint('Error: $e');
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> createSharedPcEvent(params) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClients(
              requiredToken: true, token: await appPrefs.getNormalToken())
          .post(AppEndpoint.createSharedPcEvent(), data: params);
      return NetworkResponse.fromResponse(response,
          converter: (json) => json['id']);
    } on DioError catch (e) {
      appDebugPrint('DioError: $e');
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      appDebugPrint('Error: $e');
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> getPcEventFromUser(pcEventId) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClients(
              requiredToken: true, token: await appPrefs.getNormalToken())
          .get(AppEndpoint.getPcEventFromUser(),
              queryParameters: {'pcEventId': pcEventId});
      return NetworkResponse.fromResponse(response,
          converter: (json) => PeEventModel.fromJson(json));
    } on DioError catch (e) {
      appDebugPrint('DioError: $e');
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      appDebugPrint('Error: $e');
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> setThumbnailForPcEvent(path, pcEventId) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      String fileName = path.split('/').last;
      FormData data = FormData.fromMap({
        'pcEventId': pcEventId,
        "fileName": MultipartFile.fromBytes(File(path).readAsBytesSync(),
            filename: fileName),
      });
      Response response = await AppClients(
              requiredToken: true, token: await appPrefs.getNormalToken())
          .post(AppEndpoint.setThumbnailForPcEvent(), data: data);
      return NetworkResponse.fromResponse(response,
          converter: (json) => json['fileName']);
    } on DioError catch (e) {
      appDebugPrint('DioError: $e');
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      appDebugPrint('Error: $e');
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> shareCalendarToUsers(params) =>
      throw UnimplementedError();

  @override
  Future<NetworkResponse> getHomesAroundMe(params) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClients(
              requiredToken: true, token: await appPrefs.getNormalToken())
          .get(AppEndpoint.getHomesAroundMe(), queryParameters: params);
      return NetworkResponse.fromResponse(response,
          converter: (json) => (json as List<dynamic>)
              .map<HomePlaceModel>((i) => HomePlaceModel.fromJson(i))
              .toList());
    } on DioError catch (e) {
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      appDebugPrint('Error: $e');
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> whatsNext(params) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClients(
              requiredToken: true, token: await appPrefs.getNormalToken())
          .get(AppEndpoint.whatsNext(), queryParameters: params);
      return NetworkResponse.fromResponse(response,
          converter: (json) => WhatNextModel.fromJson(json));
    } on DioError catch (e) {
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      appDebugPrint('Error: $e');
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> generateToken(channelName, expiredTime) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClients(
              requiredToken: true, token: await appPrefs.getNormalToken())
          .post(AppEndpoint.generateVideoCallToken(),
              data: {'channel': channelName});
      return NetworkResponse.fromResponse(response,
          converter: (json) => AgoraToken.fromJson(json));
    } on DioError catch (e) {
      return NetworkResponse.withErrorRequest(e);
    }
  }

  @override
  Future<NetworkResponse> getHomelidoPropertyCategories(languageCode) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClients(
              requiredToken: true, token: await appPrefs.getNormalToken())
          .get(AppEndpoint.getHomelidoPropertyCategories(),
              queryParameters: {'languageCode': languageCode});
      return NetworkResponse.fromResponse(response,
          converter: (json) => (json as List<dynamic>)
              .map<HomelidoPropertyCategory>(
                  (i) => HomelidoPropertyCategory.fromJson(i))
              .toList());
    } on DioError catch (e) {
      return NetworkResponse.withErrorRequest(e);
    }
  }

  @override
  Future<NetworkResponse> getCurrentPropertiesOfTerminal(params) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClients(
              requiredToken: true, token: await appPrefs.getNormalToken())
          .get(AppEndpoint.getCurrentPropertiesOfTerminal(),
              queryParameters: params);
      return NetworkResponse.fromResponse(response,
          converter: (json) => (json as List<dynamic>)
              .map<HomelidoProperty>((i) => HomelidoProperty.fromJson(i))
              .toList());
    } on DioError catch (e) {
      return NetworkResponse.withErrorRequest(e);
    }
  }

  @override
  Future<NetworkResponse> getPropertiesFromPro(languageCode) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClients(
              requiredToken: true, token: await appPrefs.getNormalToken())
          .get(AppEndpoint.getPropertiesFromPro(),
              queryParameters: {'languageCode': languageCode});
      return NetworkResponse.fromResponse(response,
          converter: (json) => (json as List<dynamic>)
              .map<HomelidoProperty>((i) => HomelidoProperty.fromJson(i))
              .toList());
    } on DioError catch (e) {
      return NetworkResponse.withErrorRequest(e);
    }
  }

  @override
  Future<NetworkResponse> createHomlidoMeeting(params) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClients(
              requiredToken: true, token: await appPrefs.getNormalToken())
          .post(AppEndpoint.createHomelidoMeeting(), data: params);
      return NetworkResponse.fromResponse(response,
          converter: (json) => IdResult.fromJson(json));
    } on DioError catch (e) {
      return NetworkResponse.withErrorRequest(e);
    }
  }

  @override
  Future<NetworkResponse> getHomelidoMeetingDetails(params) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClients(
              requiredToken: true, token: await appPrefs.getNormalToken())
          .get(AppEndpoint.getHomelidoMeetingDetails(),
              queryParameters: params);
      return NetworkResponse.fromResponse(response,
          converter: (json) => HomelidoMeeting.fromJson(json));
    } on DioError catch (e) {
      return NetworkResponse.withErrorRequest(e);
    }
  }

  @override
  Future<NetworkResponse> answerToHomelidoVisitRequest(params) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClients(
              requiredToken: true, token: await appPrefs.getNormalToken())
          .put(AppEndpoint.answerToHomelidoVisitRequest(), data: params);
      return NetworkResponse.fromResponse(response,
          value: response.statusCode == 200);
    } on DioError catch (e) {
      return NetworkResponse.withErrorRequest(e);
    }
  }

  @override
  Future<NetworkResponse> getNumberOfSalesFromSellers(params) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClients(
              requiredToken: true, token: await appPrefs.getNormalToken())
          .get(AppEndpoint.getNumberOfSalesFromSellers(),
              queryParameters: params);
      return NetworkResponse.fromResponse(response,
          converter: (json) => NumberOfSaleModel.fromJson(json));
    } on DioError catch (e) {
      return NetworkResponse.withErrorRequest(e);
    }
  }

  @override
  Future<NetworkResponse> setBankAccountFromUser(params) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClients(
              requiredToken: true, token: await appPrefs.getNormalToken())
          .post(AppEndpoint.setBankAccountFromUser(), data: params);
      return NetworkResponse.fromResponse(response,
          converter: (json) => BankAccountModel.fromJson(json));
    } on DioError catch (e) {
      return NetworkResponse.withErrorRequest(e);
    }
  }

  @override
  Future<NetworkResponse> updateBankInfosFromSeller(params) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClients(
              requiredToken: true, token: await appPrefs.getNormalToken())
          .put(AppEndpoint.updateBankInfosFromSeller(), data: params);
      return NetworkResponse.fromResponse(response,
          converter: (json) => BankAccountModel.fromJson(json));
    } on DioError catch (e) {
      return NetworkResponse.withErrorRequest(e);
    }
  }

  @override
  Future<NetworkResponse> getAccountTypes() async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClients(
              requiredToken: true, token: await appPrefs.getNormalToken())
          .get(AppEndpoint.getAccountTypes());
      return NetworkResponse.fromResponse(response,
          converter: (json) => (json as List)
              .map((e) => BankAccountTypeModel.fromJson(e))
              .toList());
    } on DioError catch (e) {
      return NetworkResponse.withErrorRequest(e);
    }
  }

  @override
  Future<NetworkResponse> getBankInfosFromSeller() async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClients(
              requiredToken: true, token: await appPrefs.getNormalToken())
          .get(AppEndpoint.getBankInfosFromSeller());
      return NetworkResponse.fromResponse(response,
          converter: (json) => BankAccountModel.fromJson(json));
    } on DioError catch (e) {
      return NetworkResponse.withErrorRequest(e);
    }
  }

  @override
  Future<NetworkResponse> addUserOrProToTrackingPeople(params) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClients(
              requiredToken: true, token: await appPrefs.getNormalToken())
          .post(AppEndpoint.addUserOrProToTrackingPeople(), data: params);
      return NetworkResponse.fromResponse(response,
          converter: (json) => (json as List)
              .map((e) => TrackingInfoModel.fromJson(e))
              .toList());
    } on DioError catch (e) {
      return NetworkResponse.withErrorRequest(e);
    }
  }

  @override
  Future<NetworkResponse> getTrackingInfosForMeeting(params) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClients(
              requiredToken: true, token: await appPrefs.getNormalToken())
          .get(AppEndpoint.getTrackingInfosForMeeting(),
              queryParameters: params);
      return NetworkResponse.fromResponse(response,
          converter: (json) => (json as List)
              .map((e) => TrackingInfoModel.fromJson(e))
              .toList());
    } on DioError catch (e) {
      return NetworkResponse.withErrorRequest(e);
    }
  }

  @override
  Future<NetworkResponse> updateTrackingInfosForMeeting(params) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClients(
              requiredToken: true, token: await appPrefs.getNormalToken())
          .put(AppEndpoint.updateTrackingInfosForMeeting(), data: params);
      return NetworkResponse.fromResponse(response,
          converter: (json) => (json as List)
              .map((e) => TrackingInfoModel.fromJson(e))
              .toList());
    } on DioError catch (e) {
      return NetworkResponse.withErrorRequest(e);
    }
  }

  @override
  Future<NetworkResponse> getLearnMoreTrackingVideo() async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClients(requiredToken: false).get(
          AppEndpoint.getLearnMoreTrackingVideo(),
          queryParameters: {'languageCode': appPrefs.languageCode});
      return NetworkResponse.fromResponse(response, converter: (json) => json);
    } on DioError catch (e) {
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      appDebugPrint('Error: $e');
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> getReminderTypes() async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClients(
              requiredToken: true, token: await appPrefs.getNormalToken())
          .get(AppEndpoint.getReminderTypes(),
              queryParameters: {'languageCode': appPrefs.languageCode});
      return NetworkResponse.fromResponse(response,
          converter: (json) =>
              (json as List).map((e) => ReminderType.fromJson(e)).toList());
    } on DioError catch (e) {
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      appDebugPrint('Error: $e');
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> getOowooMoveProperties(params) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClients(
              requiredToken: true, token: await appPrefs.getNormalToken())
          .get(AppEndpoint.getOowooMoveProperties(), queryParameters: params);
      return NetworkResponse.fromResponse(response,
          converter: (json) =>
              (json as List).map((e) => MoveModel.fromJson(e)).toList());
    } on DioError catch (e) {
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      appDebugPrint('Error: $e');
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> getOowooVideoByTypeId(typeId) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClients(
              requiredToken: true, token: await appPrefs.getNormalToken())
          .get(AppEndpoint.getOowooVideoByTypeId(), queryParameters: {
        'typeId': typeId,
        'languageCode': appPrefs.languageCode
      });
      return NetworkResponse.fromResponse(response,
          converter: (json) => VideoUrl.fromJson(json));
    } on DioError catch (e) {
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      appDebugPrint('Error: $e');
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> getCalmByCat(params) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClients(
              requiredToken: true, token: await appPrefs.getNormalToken())
          .get(AppEndpoint.getCalmByCat(), queryParameters: params);
      return NetworkResponse.fromResponse(response,
          converter: (json) => (json as List<dynamic>)
              .map<CalmModel>((i) => CalmModel.fromJson(i))
              .toList());
    } on DioError catch (e) {
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      appDebugPrint('Error: $e');
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> getCalmCategories() async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClients(
              requiredToken: true, token: await appPrefs.getNormalToken())
          .get(AppEndpoint.getCalmCategories(),
              queryParameters: {'languageCode': appPrefs.languageCode});
      return NetworkResponse.fromResponse(response,
          converter: (json) => (json as List<dynamic>)
              .map<CalmCate>((i) => CalmCate.fromJson(i))
              .toList());
    } on DioError catch (e) {
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      appDebugPrint('Error: $e');
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> getSubscribersForCalmId(params) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClients(
              requiredToken: true, token: await appPrefs.getNormalToken())
          .get(AppEndpoint.getSubscribersForCalmId(), queryParameters: params);
      return NetworkResponse.fromResponse(response,
          converter: (json) => (json['subscribers'] as List<dynamic>)
              .map<CalmSubscriber>((i) => CalmSubscriber.fromJson(i))
              .toList());
    } on DioError catch (e) {
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      appDebugPrint('Error: $e');
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> subscribeToCalmId(calmId) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClients(
              requiredToken: true, token: await appPrefs.getNormalToken())
          .post(AppEndpoint.subscribeToCalmId(), data: {'calmId': calmId});
      return NetworkResponse.fromResponse(response,
          value: response.statusCode == 200);
    } on DioError catch (e) {
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      appDebugPrint('Error: $e');
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> testSendNotificationToPro(params) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClients(
              requiredToken: true, token: await appPrefs.getNormalToken())
          .post(AppEndpoint.testSendNotificationToPro(), data: params);
      return NetworkResponse.fromResponse(response,
          value: response.statusCode == 200);
    } on DioError catch (e) {
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      appDebugPrint('Error: $e');
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> testSendNotificationToUser(params) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClients(
              requiredToken: true, token: await appPrefs.getNormalToken())
          .post(AppEndpoint.testSendNotificationToUser(), data: params);
      return NetworkResponse.fromResponse(response,
          value: response.statusCode == 200);
    } on DioError catch (e) {
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      appDebugPrint('Error: $e');
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> notifVideoCallProFromUser(params) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClients(
              requiredToken: true, token: await appPrefs.getNormalToken())
          .post(AppEndpoint.notifVideoCallProFromUser(), data: params);
      return NetworkResponse.fromResponse(response,
          value: response.statusCode == 200);
    } on DioError catch (e) {
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      appDebugPrint('Error: $e');
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> notifVideoCallUserFromPro(params) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClients(
              requiredToken: true, token: await appPrefs.getNormalToken())
          .post(AppEndpoint.notifVideoCallUserFromPro(), data: params);
      return NetworkResponse.fromResponse(response,
          value: response.statusCode == 200);
    } on DioError catch (e) {
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      appDebugPrint('Error: $e');
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> getReviewsFromPro(proId) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClients(
              requiredToken: true, token: await appPrefs.getNormalToken())
          .get(AppEndpoint.getReviewsFromPro(),
              queryParameters: {'proId': proId});
      return NetworkResponse.fromResponse(response,
          converter: (json) => (json as List<dynamic>)
              .map<ReviewModel>((i) => ReviewModel.fromJson(i))
              .toList());
    } on DioError catch (e) {
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      appDebugPrint('Error: $e');
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> cancelTicketByUser(
      ticketUniqueId, status, comment) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClients(
              requiredToken: true, token: await appPrefs.getNormalToken())
          .post(AppEndpoint.cancelTicketByUser(),
              data: {'ticketUniqueId': ticketUniqueId});
      return NetworkResponse.fromResponse(response,
          value: response.statusCode == 200);
    } on DioError catch (e) {
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      appDebugPrint('Error: $e');
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> confirmTicketByUser(
      ticketUniqueId, status, comment) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClients(
              requiredToken: true, token: await appPrefs.getNormalToken())
          .post(AppEndpoint.confirmTicketByUser(), data: {
        'ticketUniqueId': ticketUniqueId,
        'status': status,
        'comment': comment
      });
      return NetworkResponse.fromResponse(response,
          converter: (json) => TicketInfoModel.fromJson(json));
    } on DioError catch (e) {
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      appDebugPrint('Error: $e');
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> deleteEventsFromUser(ticketUniqueId) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClients(
              requiredToken: true, token: await appPrefs.getNormalToken())
          .delete(AppEndpoint.deleteEventsFromUser(), data: {
        'events': [ticketUniqueId]
      });
      return NetworkResponse.fromResponse(response,
          value: response.statusCode == 200);
    } on DioError catch (e) {
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      appDebugPrint('Error: $e');
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> getAllEventsFromUser() async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClients(
              requiredToken: true, token: await appPrefs.getNormalToken())
          .get(AppEndpoint.getAllEventsFromUser(),
              queryParameters: {'catLanguageCode': appPrefs.languageCode});
      return NetworkResponse.fromResponse(response,
          converter: (json) => (json as List<dynamic>)
              .map<EventInfoModel>((i) => EventInfoModel.fromJson(i))
              .toList());
    } on DioError catch (e) {
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      appDebugPrint('Error: $e');
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> getEventInfoFromUser(eventUniqueId) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClients(
              requiredToken: true, token: await appPrefs.getNormalToken())
          .get(AppEndpoint.getEventInfoFromUser(), queryParameters: {
        'catLanguageCode': appPrefs.languageCode,
        'eventUniqueId': eventUniqueId
      });
      return NetworkResponse.fromResponse(response,
          converter: (json) => EventInfoModel.fromJson(json));
    } on DioError catch (e) {
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      appDebugPrint('Error: $e');
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> getQueueInfoFromUser(ticketUniqueId) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClients(
              requiredToken: true, token: await appPrefs.getNormalToken())
          .get(AppEndpoint.getQueueInfoFromUser(), queryParameters: {
        'catLanguageCode': appPrefs.languageCode,
        'ticketUniqueId': ticketUniqueId
      });
      return NetworkResponse.fromResponse(response,
          converter: (json) => EventQueueModel.fromJson(json));
    } on DioError catch (e) {
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      appDebugPrint('Error: $e');
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> ticketUserCallBack(ticketUniqueId, duration) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClients(
              requiredToken: true, token: await appPrefs.getNormalToken())
          .post(AppEndpoint.ticketUserCallBack(),
              data: {'ticketUniqueId': ticketUniqueId, 'duration': duration});
      return NetworkResponse.fromResponse(response,
          converter: (json) => TicketInfoModel.fromJson(json));
    } on DioError catch (e) {
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      appDebugPrint('Error: $e');
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> getUserInfoByFirebaseId(firebaseId) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClients(
              requiredToken: true, token: await appPrefs.getNormalToken())
          .get(AppEndpoint.getUserInfoByFirebaseId(),
              queryParameters: {'firebaseId': firebaseId});
      return NetworkResponse.fromResponse(response,
          converter: (json) => UserModel.fromJson(json));
    } on DioError catch (e) {
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> getProInfoByFirebaseId(firebaseId) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClients(
              requiredToken: true, token: await appPrefs.getNormalToken())
          .get(AppEndpoint.getProInfoByFirebaseId(),
              queryParameters: {'firebaseId': firebaseId});
      return NetworkResponse.fromResponse(response,
          converter: (json) => ProsModel.fromJson(json));
    } on DioError catch (e) {
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> addToFavourite(proId) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClients(
              requiredToken: true, token: await appPrefs.getNormalToken())
          .post(AppEndpoint.addToFavourite(), data: {'proId': proId});
      return NetworkResponse.fromResponse(response, value: true);
    } on DioError catch (e) {
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> getFavouriteForUser() async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClients(
              requiredToken: true, token: await appPrefs.getNormalToken())
          .get(AppEndpoint.getFavouriteForUser());
      return NetworkResponse.fromResponse(response,
          converter: (json) => (json as List<dynamic>)
              .map<ProsModel>((i) => ProsModel.fromJson(i))
              .toList());
    } on DioError catch (e) {
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> removeFromFavourite(proId) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClients(
              requiredToken: true, token: await appPrefs.getNormalToken())
          .delete(AppEndpoint.removeFromFavourite(), data: {'proId': proId});
      return NetworkResponse.fromResponse(response, value: true);
    } on DioError catch (e) {
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> updateHomelidoMeeting(params) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClients(
              requiredToken: true, token: await appPrefs.getNormalToken())
          .put(AppEndpoint.updateHomelidoMeeting(), data: params);
      return NetworkResponse.fromResponse(response,
          value: response.statusCode == 200);
    } on DioError catch (e) {
      return NetworkResponse.withErrorRequest(e);
    }
  }

  @override
  Future<NetworkResponse> getPcEventFromPro(pcEventId) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClients(
              requiredToken: true, token: await appPrefs.getNormalToken())
          .get(AppEndpoint.getPcEventFromPro(),
              queryParameters: {'pcEventId': pcEventId});
      return NetworkResponse.fromResponse(response,
          converter: (json) => PeEventModel.fromJson(json));
    } on DioError catch (e) {
      appDebugPrint('DioError: $e');
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      appDebugPrint('Error: $e');
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> deleteCodeFromLog(params) {
    // TODO: implement deleteCodeFromLog
    throw UnimplementedError();
  }

  @override
  Future<NetworkResponse> createCode(params) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClients(
              requiredToken: true, token: await appPrefs.getNormalToken())
          .post(AppEndpoint.createCode(), data: params);
      return NetworkResponse.fromResponse(response,
          converter: (json) => CodeModel.fromJson(json));
    } on DioError catch (e) {
      return NetworkResponse.withErrorRequest(e);
    }
  }

  @override
  Future<NetworkResponse> updateLiveCode(params) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClients(
              requiredToken: true, token: await appPrefs.getNormalToken())
          .put(AppEndpoint.updateLiveCode(), data: params);
      return NetworkResponse.fromResponse(response,
          value: response.statusCode == 200);
    } on DioError catch (e) {
      return NetworkResponse.withErrorRequest(e);
    }
  }

  @override
  Future<NetworkResponse> shareCodeDigicode(params) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClients(
              requiredToken: true, token: await appPrefs.getNormalToken())
          .post(AppEndpoint.shareCodeDigicode(), data: params);
      return NetworkResponse.fromResponse(response,
          value: response.statusCode == 200);
    } on DioError catch (e) {
      return NetworkResponse.withErrorRequest(e);
    }
  }

  @override
  Future<NetworkResponse> deleteLiveCode(params) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClients(
              requiredToken: true, token: await appPrefs.getNormalToken())
          .delete(AppEndpoint.deleteLiveCode(), data: params);
      return NetworkResponse.fromResponse(response,
          value: response.statusCode == 200);
    } on DioError catch (e) {
      return NetworkResponse.withErrorRequest(e);
    }
  }

  @override
  Future<NetworkResponse> getCodesFromPro(digicodeId) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClients(
              requiredToken: true, token: await appPrefs.getNormalToken())
          .get(AppEndpoint.getCodesFromPro(),
              queryParameters: {'digicodeId': digicodeId});
      return NetworkResponse.fromResponse(response,
          converter: (json) =>
              (json as List).map((e) => CodeModel.fromJson(e)).toList());
    } on DioError catch (e) {
      return NetworkResponse.withErrorRequest(e);
    }
  }

  @override
  Future<NetworkResponse> getDigicodesFromPro() async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClients(
              requiredToken: true, token: await appPrefs.getNormalToken())
          .get(AppEndpoint.getDigicodesFromPro());
      return NetworkResponse.fromResponse(response,
          converter: (json) =>
              (json as List).map((e) => DigicodeModel.fromJson(e)).toList());
    } on DioError catch (e) {
      return NetworkResponse.withErrorRequest(e);
    }
  }

  @override
  Future<NetworkResponse> getDigicodeById(id) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClients(
              requiredToken: true, token: await appPrefs.getNormalToken())
          .get(AppEndpoint.getDigicodeById(), queryParameters: {'id': id});
      return NetworkResponse.fromResponse(response,
          converter: (json) => DigicodeModel.fromJson(json));
    } on DioError catch (e) {
      return NetworkResponse.withErrorRequest(e);
    }
  }

  @override
  Future<NetworkResponse> checkDigicodeBySerialAndToken(
      productSerial, authcode) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClients(
              options: BaseOptions(headers: {'auth-code': authcode}),
              requiredToken: true,
              token: await appPrefs.getNormalToken())
          .get(AppEndpoint.checkDigicodeBySerialAndToken(),
              queryParameters: {'productSerial': productSerial});
      return NetworkResponse.fromResponse(response,
          converter: (json) => DigicodeCheck.fromJson(json));
    } on DioError catch (e) {
      return NetworkResponse.withErrorRequest(e);
    }
  }

  @override
  Future<NetworkResponse> linkDigicodeToPro(params) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClients(
              requiredToken: true, token: await appPrefs.getNormalToken())
          .post(AppEndpoint.linkDigicodeToPro(), data: params);
      return NetworkResponse.fromResponse(response,
          value: response.statusCode == 200);
    } on DioError catch (e) {
      return NetworkResponse.withErrorRequest(e);
    }
  }

  @override
  Future<NetworkResponse> linkDigicodeToProV2(params) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClients(
              requiredToken: true, token: await appPrefs.getNormalToken())
          .post(AppEndpoint.linkDigicodeToProV2(), data: params);
      return NetworkResponse.fromResponse(response,
          value: response.statusCode == 200);
    } on DioError catch (e) {
      return NetworkResponse.withErrorRequest(e);
    }
  }

  @override
  Future<NetworkResponse> linkTerminalToProV2(params) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClients(
              requiredToken: true, token: await appPrefs.getNormalToken())
          .post(AppEndpoint.linkTerminalToProV2(), data: params);
      return NetworkResponse.fromResponse(response,
          value: response.statusCode == 200);
    } on DioError catch (e) {
      return NetworkResponse.withErrorRequest(e);
    }
  }

  @override
  Future<NetworkResponse> getDigicodesLog(params) {
    // TODO: implement getDigicodesLog
    throw UnimplementedError();
  }

  @override
  Future<NetworkResponse> getPlacesFromPro(params) {
    // TODO: implement getPlacesFromPro
    throw UnimplementedError();
  }

  @override
  Future<NetworkResponse> unlockDigicode(params) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClients(
              requiredToken: true, token: await appPrefs.getNormalToken())
          .post(AppEndpoint.unlockDigicode(), data: params);
      return NetworkResponse.fromResponse(response,
          value: response.statusCode == 200);
    } on DioError catch (e) {
      return NetworkResponse.withErrorRequest(e);
    }
  }

  @override
  Future<NetworkResponse> checkDigicodeOnline(digicodeId) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClients(
              requiredToken: true, token: await appPrefs.getNormalToken())
          .get(AppEndpoint.checkDigicodeOnline(),
              queryParameters: {'digicodeId': digicodeId});
      return NetworkResponse.fromResponse(response,
          prefix: 'status', converter: (json) => json);
    } on DioError catch (e) {
      return NetworkResponse.withErrorRequest(e);
    }
  }

  @override
  Future<NetworkResponse> setLanguagesForPro(proLanguageCode) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClients(
              requiredToken: true, token: await appPrefs.getNormalToken())
          .post(AppEndpoint.setLanguagesForPro(),
              data: {'proLanguageCode': proLanguageCode});
      return NetworkResponse.fromResponse(response,
          value: response.statusCode == 200);
    } on DioError catch (e) {
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      appDebugPrint('Error: $e');
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> setMainLanguageForPro(proLanguageCode) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClients(
              requiredToken: true, token: await appPrefs.getNormalToken())
          .post(AppEndpoint.setMainLanguageForPro(),
              data: {'proLanguageCode': proLanguageCode});
      return NetworkResponse.fromResponse(response,
          value: response.statusCode == 200);
    } on DioError catch (e) {
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      appDebugPrint('Error: $e');
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> getAllEventsFromPro() async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClients(
              requiredToken: true, token: await appPrefs.getNormalToken())
          .get(AppEndpoint.getAllEventsFromPro(),
              queryParameters: {'languageCode': appPrefs.languageCode});
      return NetworkResponse.fromResponse(response,
          converter: (json) =>
              (json as List).map((e) => EventInfoModel.fromJson(e)).toList());
    } on DioError catch (e) {
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      appDebugPrint('Error: $e');
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> getAllServicesOfProFromPro(params) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClients(
              requiredToken: true, token: await appPrefs.getNormalToken())
          .get(AppEndpoint.getAllServicesOfProFromPro(),
              queryParameters: params);
      return NetworkResponse.fromResponse(response,
          converter: (json) =>
              (json as List).map((e) => ProServiceModel.fromJson(e)).toList());
    } on DioError catch (e) {
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      appDebugPrint('Error: $e');
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> getAllServicesOfProFromUser(params) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClients(
              requiredToken: true, token: await appPrefs.getNormalToken())
          .get(AppEndpoint.getAllServicesOfProFromUser(),
              queryParameters: params);
      return NetworkResponse.fromResponse(response,
          converter: (json) => ProServicesBusiness.fromJson(json));
    } on DioError catch (e) {
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      appDebugPrint('Error: $e');
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> getAllServicesAroundMe(params) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClients(
              requiredToken: true, token: await appPrefs.getNormalToken())
          .get(AppEndpoint.getAllServicesAroundMe(), queryParameters: params);
      return NetworkResponse.fromResponse(response,
          converter: (json) => (json as List)
              .map((e) => ServicesAroundMeModel.fromJson(e))
              .toList());
    } on DioError catch (e) {
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      appDebugPrint('Error: $e');
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> createTicketByUser(params) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClients(
              requiredToken: true, token: await appPrefs.getNormalToken())
          .post(AppEndpoint.createTicketByUser(), data: params);
      return NetworkResponse.fromResponse(response,
          converter: (json) => TicketInfoModel.fromJson(json));
    } on DioError catch (e) {
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> callNextTicket(serviceId) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClients(
              requiredToken: true, token: await appPrefs.getNormalToken())
          .post(AppEndpoint.callNextTicket(), data: {'serviceId': serviceId});
      return NetworkResponse.fromResponse(response,
          converter: (json) => TicketInfoModel.fromJson(json));
    } on DioError catch (e) {
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> cancelTicketByPro(ticketUniqueId) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClients(
              requiredToken: true, token: await appPrefs.getNormalToken())
          .post(AppEndpoint.cancelTicketByPro(),
              data: {'ticketUniqueId': ticketUniqueId});
      return NetworkResponse.fromResponse(response,
          value: response.statusCode == 200);
    } on DioError catch (e) {
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> canceledByProAfterCallBack(ticketUniqueId) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClients(
              requiredToken: true, token: await appPrefs.getNormalToken())
          .post(AppEndpoint.canceledByProAfterCallBack(),
              data: {'ticketUniqueId': ticketUniqueId});
      return NetworkResponse.fromResponse(response,
          converter: (json) => EventInfoModel.fromJson(json));
    } on DioError catch (e) {
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> confirmTicketByPro(
      ticketUniqueId, status, comment) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClients(
              requiredToken: true, token: await appPrefs.getNormalToken())
          .post(AppEndpoint.confirmTicketByPro(), data: {
        'ticketUniqueId': ticketUniqueId,
        'confirm': status,
        'comment': comment
      });
      return NetworkResponse.fromResponse(response,
          converter: (json) => EventInfoModel.fromJson(json));
    } on DioError catch (e) {
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> createTicketByPro(params) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClients(
              requiredToken: true, token: await appPrefs.getNormalToken())
          .post(AppEndpoint.createTicketByPro(), data: params);
      return NetworkResponse.fromResponse(response,
          converter: (json) => TicketInfoModel.fromJson(json));
    } on DioError catch (e) {
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> declareUserAsAbsentFromPro(ticketUniqueId) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClients(
              requiredToken: true, token: await appPrefs.getNormalToken())
          .post(AppEndpoint.declareUserAsAbsentFromPro(),
              data: {'ticketUniqueId': ticketUniqueId});
      return NetworkResponse.fromResponse(response,
          converter: (json) => EventInfoModel.fromJson(json));
    } on DioError catch (e) {
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> deleteEventsFromPro(ticketUniqueId) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClients(
              requiredToken: true, token: await appPrefs.getNormalToken())
          .delete(AppEndpoint.deleteEventsFromPro(), data: {
        'events': [ticketUniqueId]
      });
      return NetworkResponse.fromResponse(response,
          converter: (json) => EventInfoModel.fromJson(json));
    } on DioError catch (e) {
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> getEventInfoFromPro(eventUniqueId) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClients(
              requiredToken: true, token: await appPrefs.getNormalToken())
          .get(AppEndpoint.getEventInfoFromPro(),
              queryParameters: {'eventUniqueId': eventUniqueId});
      return NetworkResponse.fromResponse(response,
          converter: (json) => EventInfoModel.fromJson(json));
    } on DioError catch (e) {
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> getListTimeFramesByServiceForDay(
      dateToday, serviceId) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClients(
              requiredToken: true, token: await appPrefs.getNormalToken())
          .get(AppEndpoint.getListTimeFramesByServiceForDay(),
              queryParameters: {
            'dateToday': dateToday,
            'serviceId': serviceId
          });
      return NetworkResponse.fromResponse(response,
          converter: (json) => ListTimeFrameModel.fromJson(json));
    } on DioError catch (e) {
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> getServiceAvailabilityForMonth(params) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClients(
              requiredToken: true, token: await appPrefs.getNormalToken())
          .get(AppEndpoint.getServiceAvailabilityForMonth(),
              queryParameters: params);
      return NetworkResponse.fromResponse(response,
          converter: (json) => (json as List)
              .map((e) => ServiceAvailabilityModel.fromJson(e))
              .toList());
    } on DioError catch (e) {
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> startTicket(ticketUniqueId) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClients(
              requiredToken: true, token: await appPrefs.getNormalToken())
          .post(AppEndpoint.startTicket(),
              data: {'ticketUniqueId': ticketUniqueId});
      return NetworkResponse.fromResponse(response,
          value: response.statusCode == 200);
    } on DioError catch (e) {
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> stopTicket(ticketUniqueId) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClients(
              requiredToken: true, token: await appPrefs.getNormalToken())
          .post(AppEndpoint.stopTicket(),
              data: {'ticketUniqueId': ticketUniqueId});
      return NetworkResponse.fromResponse(response,
          value: response.statusCode == 200);
    } on DioError catch (e) {
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> ticketUserNotResponse(ticketUniqueId) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClients(
              requiredToken: true, token: await appPrefs.getNormalToken())
          .post(AppEndpoint.ticketUserNotResponse(),
              data: {'ticketUniqueId': ticketUniqueId});
      return NetworkResponse.fromResponse(response,
          value: response.statusCode == 200);
    } on DioError catch (e) {
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> updateDelay(params) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClients(
              requiredToken: true, token: await appPrefs.getNormalToken())
          .post(AppEndpoint.updateDelay(), data: params);
      return NetworkResponse.fromResponse(response,
          converter: (json) => (json as List)
              .map((e) => ServiceDelayModel.fromJson(e))
              .toList());
    } on DioError catch (e) {
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> getLoginSessions(params) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClients(
              requiredToken: true, token: await appPrefs.getNormalToken())
          .get(AppEndpoint.getLoginSessions(), queryParameters: params);
      return NetworkResponse.fromResponse(response,
          converter: (json) =>
              (json as List).map((e) => LoginSession.fromJson(e)).toList());
    } on DioError catch (e) {
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> revokeLoginSession(params) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClients(
              requiredToken: true, token: await appPrefs.getNormalToken())
          .post(AppEndpoint.revokeLoginSession(), data: params);
      return NetworkResponse.fromResponse(response,
          value: response.statusCode == 200);
    } on DioError catch (e) {
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> allowPendingLoginOnPlatformByQrCode(params) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClients(
              requiredToken: true, token: await appPrefs.getNormalToken())
          .post(AppEndpoint.allowPendingLoginOnPlatformByQrCode(),
              data: params);
      return NetworkResponse.fromResponse(response,
          value: response.statusCode == 200);
    } on DioError catch (e) {
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> allowPendingLoginOnPlatformById(params) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClients(
              requiredToken: true, token: await appPrefs.getNormalToken())
          .post(AppEndpoint.allowPendingLoginOnPlatformById(), data: params);
      return NetworkResponse.fromResponse(response,
          value: response.statusCode == 200);
    } on DioError catch (e) {
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> activatePourDevice(deviceId) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClients(
              requiredToken: true, token: await appPrefs.getNormalToken())
          .post(AppEndpoint.activatePourDevice(), data: {'deviceId': deviceId});
      return NetworkResponse.fromResponse(response,
          value: response.statusCode == 200);
    } on DioError catch (e) {
      print('DioError: $e');
      if (e.response?.statusCode == 400)
        return NetworkResponse.fromResponse(e.response!,
            value: e.response!.data['error']);
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      print('Error: $e');
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> createPurchaseByCardOnFileStripe(params) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClients(
              requiredToken: true, token: await appPrefs.getNormalToken())
          .post(AppEndpoint.createPurchaseByCardOnFileStripe(), data: params);
      return NetworkResponse.fromResponse(response,
          converter: (json) => (json as List<dynamic>)
              .map<PurchaseInfo>((i) => PurchaseInfo.fromJson(i))
              .toList());
    } on DioError catch (e) {
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> getBgVideo(countryCode) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      final token = await appPrefs.getNormalToken();
      Response response = await AppClients(requiredToken: true, token: token)
          .get(AppEndpoint.getBgVideo(),
              queryParameters: {'countryCode': countryCode});
      return NetworkResponse.fromResponse(response,
          converter: (json) => json['videoUrl']);
    } on DioError catch (e) {
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      print('Error: $e');
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> getCurrencyAvailableForToken(tokenType) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClients(requiredToken: false).get(
          AppEndpoint.getCurrencyAvailableForToken(),
          queryParameters: {'tokenType': tokenType});
      return NetworkResponse.fromResponse(response,
          converter: (json) =>
              (json as List<dynamic>).map<String>((i) => i).toList());
    } on DioError catch (e) {
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> getCurrentTokenValue(tokenType,
      {currency = 'EUR'}) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClients(requiredToken: false).get(
          AppEndpoint.getCurrentTokenValue(),
          queryParameters: {'tokenType': tokenType, 'currency': currency});
      return NetworkResponse.fromResponse(response,
          converter: (json) => double.parse(json.toString()));
    } on DioError catch (e) {
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> getHistoryFromUser() async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      final token = await appPrefs.getNormalToken();
      Response response = await AppClients(requiredToken: true, token: token)
          .get(AppEndpoint.getHistoryFromUser());
      return NetworkResponse.fromResponse(response,
          converter: (json) => (json as List<dynamic>)
              .map<History>((i) => History.fromJson(i))
              .toList());
    } on DioError catch (e) {
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> getPmbDevicesAroundMe(params) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      final token = await appPrefs.getNormalToken();
      Response response = await AppClients(requiredToken: true, token: token)
          .get(AppEndpoint.getPmbDevicesAroundMe(), queryParameters: params);
      return NetworkResponse.fromResponse(response,
          converter: (json) => (json as List<dynamic>)
              .map<PmbDevice>((i) => PmbDevice.fromJson(i))
              .toList());
    } on DioError catch (e) {
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      print('Error: $e');
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> getWalletInfo() async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response = await AppClients(
              requiredToken: true, token: await appPrefs.getNormalToken())
          .get(AppEndpoint.getWalletInfo());
      return NetworkResponse.fromResponse(response,
          converter: (json) => WalletInfo.fromJson(json));
    } on DioError catch (e) {
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> ipAPI() async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      Response response =
          await AppClients(requiredToken: false, baseUrl: 'http://ip-api.com/')
              .get('json');
      return NetworkResponse.fromResponse(response,
          converter: (json) => IpInfo.fromJson(json), prefix: '');
    } on DioError catch (e) {
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      print('Error: $e');
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> scanDeviceQr(params) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      final token = await appPrefs.getNormalToken();
      Response response = await AppClients(requiredToken: true, token: token)
          .get(AppEndpoint.scanDeviceQr(), queryParameters: params);
      return NetworkResponse.fromResponse(response,
          converter: (json) => ScanQr.fromJson(json));
    } on DioError catch (e) {
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      print('Error: $e');
      return NetworkResponse.withErrorConvert(e);
    }
  }

  @override
  Future<NetworkResponse> checkAndAddNewContact(params) async {
    bool isDisconnect = await WifiService.isDisconnectWhenCallApi();
    if (isDisconnect) return NetworkResponse.withDisconnect();
    try {
      final token = await appPrefs.getNormalToken();
      Response response = await AppClients(requiredToken: true, token: token)
          .post(AppEndpoint.checkAndAddNewContact(), data: params);
      return NetworkResponse.fromResponse(response,
          value: response.statusCode == 200);
    } on DioError catch (e) {
      return NetworkResponse.withErrorRequest(e);
    } catch (e) {
      print('Error: $e');
      return NetworkResponse.withErrorConvert(e);
    }
  }
}
