import 'package:oowoo_core/oowoo_core.dart';

import '../model/google_place/details_response.dart';
import '../model/google_place/autocomplete_prediction.dart';
import '../resources.dart';

class BasicRepo {
  BasicRepo._();

  static BasicRepo? _instance;

  factory BasicRepo([AppApi? api]) {
    _instance ??= BasicRepo._();
    _instance!._api = api ?? AppApiImp();
    return _instance!;
  }

  late AppApi _api;

  Future<List<DateFormatModel>> getDateFormatList() async {
    NetworkResponse response = await _api.getDateFormatList();
    return response.data ?? [];
  }

  Future<List<TimeFormatModel>> getTimeFormatList() async {
    NetworkResponse response = await _api.getTimeFormatList();
    return response.data ?? [];
  }

  Future<List<Country>> getPhoneAvailable() async {
    NetworkResponse response = await _api.getPhoneAvailable();
    return response.data ?? [];
  }

  Future<List<Country>> getCountryAvailable() async {
    NetworkResponse response = await _api.getCountryAvailable();
    return response.data ?? [];
  }

  Future<List<LanguageModel>> getLanguageAvailable() async {
    NetworkResponse response = await _api.getLanguageAvailable();
    return response.data ?? [];
  }

  Future<List<LanguageModel>> getLanguageForSpeakAvailable() async {
    NetworkResponse response = await _api.getLanguageForSpeakAvailable();
    return response.data ?? [];
  }

  Future<List<AutocompletePrediction>> googlePlaceAutoComplete(
      String keyword, String countryCode,
      [String? types]) async {
    NetworkResponse response =
        await _api.googlePlaceAutoComplete(keyword, countryCode, types);
    return response.data ?? [];
  }

  Future<DetailsResponse?> googlePlaceSearchById(String placeId) async {
    NetworkResponse response = await _api.googlePlaceSearchById(placeId);
    return response.data;
  }

  Future<VideoUrl?> getOowooVideoByTypeId(typeId) async {
    NetworkResponse response = await _api.getOowooVideoByTypeId(typeId);
    return response.data;
  }

  Future<List<AllowedFeatureModel>> getAllowedFeaturesForOowooApp() async {
    NetworkResponse response = await _api.getAllowedFeaturesForOowooApp();
    return response.data ?? [];
  }

  Future<List<DashboardInfo>> getDashboardInfos() async {
    NetworkResponse response = await _api.getDashboardInfos();
    return response.data ?? [];
  }

  Future<String?> getTncOowooApp() async {
    NetworkResponse response = await _api.getTncOowooApp();
    return response.data;
  }

  Future<String?> getTncOowooSeller() async {
    NetworkResponse response = await _api.getTncOowooSeller();
    return response.data;
  }

  Future<String?> getLearnMoreTrackingVideo() async {
    NetworkResponse response = await _api.getLearnMoreTrackingVideo();
    return response.data;
  }

  Future<bool> testSendNotificationToPro(
      {required id, required title, required body, required data}) async {
    NetworkResponse response = await _api.testSendNotificationToPro({
      "body": data,
      "title": title,
      "message": body,
      "dashboard": "oowoo",
      "pro_id": id
    });
    return response.data ?? false;
  }

  Future<bool> testSendNotificationToUser(
      {required id, required title, required body, required data}) async {
    NetworkResponse response = await _api.testSendNotificationToUser({
      "body": data,
      "title": title,
      "message": body,
      "dashboard": "oowoo",
      "user_id": id
    });
    return response.data ?? false;
  }

  Future<bool> notifVideoCallProFromUser(
      {required proId, required ticketUniqueId, required notifType}) async {
    NetworkResponse response = await _api.notifVideoCallProFromUser({
      "proId": proId,
      "ticketUniqueId": ticketUniqueId,
      "notifType": notifType
    });
    return response.data ?? false;
  }

  Future<bool> notifVideoCallUserFromPro(
      {required userId, required ticketUniqueId, required notifType}) async {
    NetworkResponse response = await _api.notifVideoCallUserFromPro({
      "userId": userId,
      "ticketUniqueId": ticketUniqueId,
      "notifType": notifType
    });
    return response.data ?? false;
  }

  Future<List<IndustryModel>> getIndustriesList() async {
    NetworkResponse response = await _api.getIndustriesList();
    return response.data ?? [];
  }

  Future<String?> getBgVideo() async {
    NetworkResponse response = await _api.getBgVideo(appPrefs.languageCode);
    return response.data ?? [];
  }

  Future<dynamic> activatePourDevice(deviceId) async {
    NetworkResponse response = await _api.activatePourDevice(deviceId);
    return response.data;
  }

  Future<IpInfo?> ipAPI() async {
    NetworkResponse response = await _api.ipAPI();
    return response.data;
  }

  Future<List<String>> getCurrencyAvailableForToken(tokenType) async {
    NetworkResponse response =
        await _api.getCurrencyAvailableForToken(tokenType);
    return response.data ?? [];
  }

  Future<double> getCurrentTokenValue(tokenType, currency) async {
    NetworkResponse response =
        await _api.getCurrentTokenValue(tokenType, currency: currency);
    return response.data ?? 0.0;
  }
}
