import 'package:oowoo_core/oowoo_constant.dart';
import 'package:oowoo_core/extensions/date_extension.dart';

import '../resources.dart';

class OtherRepository {
  OtherRepository._();

  static OtherRepository? _instance;

  factory OtherRepository([AppApi? api]) {
    _instance ??= OtherRepository._();
    _instance!._api = api ?? AppApiImp();
    return _instance!;
  }

  late AppApi _api;

  Future<List<OnThisDayModel>> birthsOnThisDay(DateTime date) async {
    String _ = '${date.month}/${date.day}';
    NetworkResponse response = await _api.birthsOnThisDay(_);
    return response.data ?? [];
  }

  Future<List<OnThisDayModel>> deathsOnThisDay(DateTime date) async {
    String _ = '${date.month}/${date.day}';
    NetworkResponse response = await _api.deathsOnThisDay(_);
    return response.data ?? [];
  }

  Future<List<OnThisDayModel>> eventsOnThisDay(DateTime date) async {
    String _ = '${date.month}/${date.day}';
    NetworkResponse response = await _api.eventsOnThisDay(_);
    return response.data ?? [];
  }

  Future<List<PexelPhotoModel>> pexelsSearch(
      {String? query,
      String? orientation,
      String? size,
      int per_page = 10}) async {
    NetworkResponse response = await _api.pexelsSearch({
      'query': query ?? 'galaxy',
      'orientation': orientation ?? 'landscape',
      'size': size ?? 'small',
      'per_page': per_page
    });
    return response.data ?? [];
  }

  Future<WeatherForecast?> weatherForecast(
      {double? lat, double? lng, String? location}) async {
    Map<String, dynamic> params = {
      'key': WEATHER_API_KEY,
      'days': 10,
      'aqi': 'no',
      'alerts': 'no'
    };
    if (location != null && location.trim().isNotEmpty) {
      params.addAll({'q': location});
    } else if (lat != null && lng != null) {
      params.addAll({'q': '$lat,$lng'});
    }
    NetworkResponse response = await _api.weatherForecast(params);
    return response.data;
  }

  Future<WeatherForecast?> weatherHistory(
      {required DateTime date,
      double? lat,
      double? lng,
      String? location}) async {
    Map<String, dynamic> params = {
      'key': WEATHER_API_KEY,
      'dt': date.formatDate(formatType: 'yyyy-MM-dd'),
      'aqi': 'no',
      'alerts': 'no'
    };
    if (location != null && location.trim().isNotEmpty) {
      params.addAll({'q': location});
    } else if (lat != null && lng != null) {
      params.addAll({'q': '$lat,$lng'});
    }
    NetworkResponse response = await _api.weatherHistory(params);
    return response.data;
  }

  Future<List<SeachPlaceResultModel>> googlePlaceSeachPlaceByText(
      String keyword,
      [types]) async {
    NetworkResponse response =
        await _api.googlePlaceSeachPlaceByText(keyword, types);
    return response.data ?? [];
  }

  Future<SeachPlaceResultDetailModel?> googlePlaceSeachPlaceDetailById(
      placeId) async {
    NetworkResponse response =
        await _api.googlePlaceSeachPlaceDetailById(placeId);
    return response.data;
  }

  Future<List<SeachPlaceResultDetailModel>> googleGeocode(latlng) async {
    NetworkResponse response = await _api.googleGeocode(latlng);
    return response.data ?? [];
  }
}
