import '../resources.dart';
import 'package:oowoo_core/network_resources/model/model.dart';

class PaymentRepo {
  PaymentRepo._();

  static PaymentRepo? _instance;

  factory PaymentRepo([AppApi? api]) {
    if (_instance == null) _instance = PaymentRepo._();
    _instance!._api = api ?? AppApiImp();
    return _instance!;
  }

  late AppApi _api;

  Future<List<CardStripeModel>> getCustomerCardsStripe(page, pageSize) async {
    NetworkResponse response =
        await _api.getCustomerCardsStripe(page, pageSize);
    return response.data ?? [];
  }

  Future<CardStripeModel?> createCustomerCardStripe(params) async {
    NetworkResponse response = await _api.createCustomerCardStripe(params);
    return response.data;
  }

  Future<bool> deleteCustomerCardStripe(cardId) async {
    NetworkResponse response = await _api.deleteCustomerCardStripe(cardId);
    return response.data ?? false;
  }

  Future<bool> setCustomerCardDefaultStripe(cardId) async {
    NetworkResponse response = await _api.setCustomerCardDefaultStripe(cardId);
    return response.data ?? false;
  }

  Future<WalletInfo?> getWalletInfo() async {
    NetworkResponse response = await _api.getWalletInfo();
    return response.data;
  }

  Future<List<PurchaseInfo>> createPurchaseByCardOnFileStripe(
      {required currency,
      required tokenType,
      required priceByUnit,
      required cardId,
      required amount}) async {
    var params = {
      'purchaseType': 90,
      'currency': currency,
      'tokenType': tokenType,
      'priceByUnit': priceByUnit,
      'amount': amount,
      'cardId': cardId,
    };
    NetworkResponse response =
        await _api.createPurchaseByCardOnFileStripe(params);
    return response.data ?? [];
  }
}
