import 'package:flutter/foundation.dart';
import 'package:flutter/material.dart';

@immutable
class OOWOOThemeData with Diagnosticable {
  factory OOWOOThemeData({
    Color? bottomSheetGripColor,
    Color? bottomSheetBacgroudColor,
    BoxDecoration Function(bool)? buttonDecoration,
    Widget Function()? imageLoginBuilder,
    Widget Function(Widget)? chatDetailBackgroundBuilder,
    Widget Function(Widget child, dynamic data, dynamic timestamp)?
        chatDetailContentActionBuilder,
    Widget Function(bool enableButtonBack, dynamic state)?
        chatDetailProfileBuilder,
    Widget Function(dynamic state, TextEditingController controller,
            FocusNode node, Function() onSend)?
        chatDetailInputBuilder,
    double? chatBubbleMaxWidth,
    EdgeInsets? chatBubbleMarginContent,
    TextStyle? chatTextStyleMessageState,
    Widget Function(String content, bool isMe, Map data, dynamic indexOfGroup)?
        chatTextContentBuilder,
    Widget Function(String content, bool isMe, Map data, dynamic indexOfGroup)?
        chatAudioContentBuilder,
    Color? chatBackgroundColorDetail,
    double? chatBubbleMaxWidthAppointment,
    Widget Function(Widget seenState, String thumbnail, String fullname,
            List<Widget> messages, bool isMe)?
        chatOwnerBubbleBuilder,
    Widget Function(bool chatListLoading, bool chatDetailLoading)?
        chatDetailLoadingBuilder,
    Widget Function(String asset, int count)? chatReactionIconBuilder,
    Widget Function(Map data, String imageUrl, String fullName)?
        chatTextReplyMessageBuilder,
    bool? enableReplyMessageOutsideBubble,
    Function(ImageProvider imageProvider, Map data)? chatOnOpenImage,
    List<Widget> Function(List messages)? chatDetailGroupedMessagesBuilder,
    String? loginVideoBackgroundAsset,
    Widget Function(BuildContext)? loginGetStartedBuilder,
    double? loginBorderRadius,
  }) {
    return OOWOOThemeData.raw(
      bottomSheetGripColor: bottomSheetGripColor,
      bottomSheetBacgroudColor: bottomSheetBacgroudColor,
      loginButtonDecoration: buttonDecoration,
      loginImageBannerBuilder: imageLoginBuilder,
      chatDetailProfileBuilder: chatDetailProfileBuilder,
      chatDetailInputBuilder: chatDetailInputBuilder,
      chatBackgroundColorDetail: chatBackgroundColorDetail,
      chatBubbleMaxWidth: chatBubbleMaxWidth,
      chatBubbleMaxWidthAppointment: chatBubbleMaxWidthAppointment,
      chatBubbleMarginContent: chatBubbleMarginContent,
      chatTextStyleMessageState: chatTextStyleMessageState,
      chatTextContentBuilder: chatTextContentBuilder,
      chatAudioContentBuilder: chatAudioContentBuilder,
      chatOwnerBubbleBuilder: chatOwnerBubbleBuilder,
      chatReactionIconBuilder: chatReactionIconBuilder,
      chatDetailBackgroundBuilder: chatDetailBackgroundBuilder,
      chatDetailContentActionBuilder: chatDetailContentActionBuilder,
      chatTextReplyMessageBuilder: chatTextReplyMessageBuilder,
      enableReplyMessageOutsideBubble: enableReplyMessageOutsideBubble,
      chatDetailLoadingBuilder: chatDetailLoadingBuilder,
      chatOnOpenImage: chatOnOpenImage,
      chatDetailGroupedMessagesBuilder: chatDetailGroupedMessagesBuilder,
      loginGetStartedBuilder: loginGetStartedBuilder,
      loginVideoBackgroundAsset: loginVideoBackgroundAsset,
      loginBorderRadius: loginBorderRadius,
    );
  }

  const OOWOOThemeData.raw({
    this.chatDetailBackgroundBuilder,
    this.chatDetailContentActionBuilder,
    this.bottomSheetGripColor,
    this.bottomSheetBacgroudColor,
    this.loginButtonDecoration,
    this.loginImageBannerBuilder,
    this.chatDetailProfileBuilder,
    this.chatDetailInputBuilder,
    this.chatBubbleMarginContent,
    this.chatBubbleMaxWidth,
    this.chatBubbleMaxWidthAppointment,
    this.chatTextContentBuilder,
    this.chatAudioContentBuilder,
    this.chatTextStyleMessageState,
    this.chatBackgroundColorDetail,
    this.chatOwnerBubbleBuilder,
    this.chatReactionIconBuilder,
    this.chatTextReplyMessageBuilder,
    this.enableReplyMessageOutsideBubble,
    this.chatDetailLoadingBuilder,
    this.chatOnOpenImage,
    this.chatDetailGroupedMessagesBuilder,
    this.loginGetStartedBuilder,
    this.loginBorderRadius,
    this.loginVideoBackgroundAsset,
  });

  final Color? bottomSheetGripColor;
  final Color? bottomSheetBacgroudColor;

  //Login plugin
  final BoxDecoration Function(bool)? loginButtonDecoration;
  final Widget Function()? loginImageBannerBuilder;

  //Login mobile
  final double? loginBorderRadius;
  final String? loginVideoBackgroundAsset;
  final Widget Function(BuildContext)? loginGetStartedBuilder;

  //Chat
  final double? chatBubbleMaxWidth;
  final double? chatBubbleMaxWidthAppointment;

  //UI chat detail screen
  final Widget Function(bool chatListLoading, bool chatDetailLoading)?
      chatDetailLoadingBuilder;
  final Widget Function(Widget)? chatDetailBackgroundBuilder;
  final Color? chatBackgroundColorDetail;
  final Widget Function(bool enableButtonBack, dynamic state)?
      chatDetailProfileBuilder;
  final Widget Function(dynamic state, TextEditingController controller,
      FocusNode node, Function() onSend)? chatDetailInputBuilder;

  //Message state styles
  final TextStyle? chatTextStyleMessageState;

  //Message content styles
  final EdgeInsets? chatBubbleMarginContent;
  final List<Widget> Function(List messages)? chatDetailGroupedMessagesBuilder;
  final Widget Function(Widget child, dynamic data, dynamic timestamp)?
      chatDetailContentActionBuilder;
  final Widget Function(
          String content, bool isMe, Map data, dynamic indexOfGroup)?
      chatTextContentBuilder;
  final Widget Function(
          String content, bool isMe, Map data, dynamic indexOfGroup)?
      chatAudioContentBuilder;
  final Widget Function(Map data, String imageUrl, String fullName)?
      chatTextReplyMessageBuilder;
  final bool? enableReplyMessageOutsideBubble;

  //Owner infos and messages display
  final Widget Function(Widget seenState, String thumbnail, String fullname,
      List<Widget> messages, bool isMe)? chatOwnerBubbleBuilder;

  //Reaction icon display builder
  final Widget Function(String asset, int count)? chatReactionIconBuilder;

  //Open message type image
  final Function(ImageProvider imageProvider, Map data)? chatOnOpenImage;
}

class OOWOOTheme extends InheritedTheme {
  const OOWOOTheme({Key? key, required this.data, required this.child})
      : super(key: key, child: child);

  final OOWOOThemeData data;

  @override
  final Widget child;

  static OOWOOThemeData of(BuildContext context) {
    final OOWOOTheme? sfCalendarTheme =
        context.dependOnInheritedWidgetOfExactType<OOWOOTheme>();
    return sfCalendarTheme!.data;
  }

  @override
  bool updateShouldNotify(OOWOOTheme oldWidget) => data != oldWidget.data;

  @override
  Widget wrap(BuildContext context, Widget child) {
    final OOWOOTheme? ancestorTheme =
        context.findAncestorWidgetOfExactType<OOWOOTheme>();
    return identical(this, ancestorTheme)
        ? child
        : OOWOOTheme(data: data, child: child);
  }
}
