import 'dart:ui';
import 'dart:ui' as ui;

import 'package:flutter/foundation.dart';
import 'package:flutter/material.dart';
import 'package:flutter/services.dart';
import 'package:get/get.dart';
import 'package:intl/intl.dart';

import '../oowoo_core.dart';

void oowooHaptic() {
  HapticFeedback.lightImpact();
}

class OOWOOUtils {
  OOWOOUtils._();
  static var pixelRatio = window.devicePixelRatio;

  //Size in physical pixels
  static var physicalScreenSize = window.physicalSize;
  static var physicalWidth = physicalScreenSize.width;
  static var physicalHeight = physicalScreenSize.height;

  //Size in logical pixels
  static var logicalScreenSize = window.physicalSize / pixelRatio;
  static var logicalWidth = logicalScreenSize.width;
  static var logicalHeight = logicalScreenSize.height;
}

String assetpng(x) => 'assets/images/png/$x.png';

String assetjpg(x) => 'assets/images/jpg/$x.jpg';

String assetsvg(x) => 'assets/images/svg/$x.svg';

String assetlottie(x) => 'assets/lotties/$x.json';

String assetjson(x) => 'assets/json/$x.json';

String appImageCorrectUrl(String url, {base}) {
  if (url.trim().indexOf('http') != 0) {
    return (base ?? oowoobaseUrl ?? '') + url;
  }
  return url;
}

appDebugPrint(m) {
  if (kDebugMode) {
    if (m is! String) {
      print(m);
    } else {
      debugPrint(m);
    }
  }
}

Size textSize({required text, required context, required style}) {
  return (TextPainter(
          text: TextSpan(text: text, style: style),
          maxLines: 1,
          textScaleFactor: MediaQuery.of(context).textScaleFactor,
          textDirection: ui.TextDirection.ltr)
        ..layout())
      .size;
}

hideKeyboard() {
  final currentFocus = FocusScope.of(Get.context!);
  if (!currentFocus.hasPrimaryFocus && currentFocus.hasFocus) {
    FocusManager.instance.primaryFocus?.unfocus();
  }
}

T valueByPlatform<T>(OOWOOPlatform platform, kdefault, {khomelido}) {
  switch (platform) {
    case OOWOOPlatform.homelidoPro:
      return khomelido;
    default:
      return kdefault;
  }
}

T valueByPlatformSetting<T>(kdefault, {khomelido}) {
  switch (OOWOOConfig.of(Get.context!).oowooPlatform) {
    case OOWOOPlatform.homelidoPro:
      return khomelido;
    default:
      return kdefault;
  }
}

int valuePlatformByConfig() {
  switch (OOWOOConfig.of(Get.context!).oowooPlatform) {
    case OOWOOPlatform.master:
      return master;
    case OOWOOPlatform.mobileUser:
      return oowooAppUser;
    case OOWOOPlatform.mobilePro:
      return oowooAppPro;
    case OOWOOPlatform.pluginWebUser:
      return oowooAppUsePlugin;
    case OOWOOPlatform.pluginWebPro:
      return proDashboard;
    case OOWOOPlatform.homelidoPro:
      return homelidoPro;
    case OOWOOPlatform.webPolisinUser:
      return websiteUSERPolisiN;
    case OOWOOPlatform.proDashboardArckipelFullVersion:
      return ProDashboardArckipelFullVersion;
    default:
      return master;
  }
}

OOWOOPlatform platformTypeById(platformId) {
  switch (platformId) {
    case master:
      return OOWOOPlatform.master;
    case oowooAppUser:
      return OOWOOPlatform.mobileUser;
    case oowooAppPro:
      return OOWOOPlatform.mobilePro;
    case oowooAppUsePlugin:
      return OOWOOPlatform.pluginWebUser;
    case proDashboard:
      return OOWOOPlatform.pluginWebPro;
    case homelidoPro:
      return OOWOOPlatform.homelidoPro;
    case websiteUSERPolisiN:
      return OOWOOPlatform.webPolisinUser;
    case ProDashboardArckipelFullVersion:
      return OOWOOPlatform.proDashboardArckipelFullVersion;
    default:
      return OOWOOPlatform.master;
  }
}

String currencySymbol(currency) {
  if (currency == null) return '';
  try {
    return NumberFormat.simpleCurrency(locale: currency).currencySymbol;
  } catch (e) {}
  try {
    return NumberFormat.simpleCurrency(name: currency).currencySymbol;
  } catch (e) {}
  return '';
}

Color hexColor(String hexColor) {
  try {
    hexColor = hexColor.toUpperCase().replaceAll('#', '');
    if (hexColor.length == 6) {
      hexColor = 'FF$hexColor';
    }
    return Color(int.parse(hexColor, radix: 16));
  } catch (ex) {
    return Colors.white;
  }
}

DateTime? convertString2DateTime(String? dateTime,
    {String format = "yyyy-MM-ddTHH:mm:ss.SSSZ"}) {
  if (dateTime == null) return null;
  return DateFormat(format).parse(dateTime);
}

String colorToHex(Color color) {
  return '#${color.value.toRadixString(16).padLeft(8, '0')}';
}

DateTime fromTimestamp(int seconds) {
  return DateTime.fromMillisecondsSinceEpoch(seconds * 1000);
}

showDialogAsBottomSheet({required Widget child, barrierDismissible = false}) {
  return showGeneralDialog(
    barrierColor: Colors.black12,
    transitionBuilder: (context, anim1, __, _) {
      return SlideTransition(
        position:
            Tween(begin: const Offset(0, 1), end: const Offset(0, 0)).animate(
          CurvedAnimation(
            parent: anim1,
            curve: const Cubic(0.68, -0.55, 0.265, 1.15),
            reverseCurve: Curves.ease,
          ),
        ),
        child: Opacity(
          opacity: anim1.value,
          child: child,
        ),
      );
    },
    transitionDuration: const Duration(milliseconds: 350),
    barrierDismissible: barrierDismissible,
    barrierLabel: '',
    context: Get.context!,
    pageBuilder: (context, _, __) => Container(),
  );
}
