import 'dart:async';

import 'package:blur/blur.dart';
import 'package:get/get.dart';
import 'package:mobile_scanner/mobile_scanner.dart';
import 'package:oowoo_core/oowoo_core.dart';
import 'package:flutter/material.dart';
import 'package:flutter/services.dart';
import 'package:lottie/lottie.dart';

import 'package:oowoo_core/network_resources/resources.dart';

import '../widget_glass_background.dart';

class QrCodeScreen extends StatefulWidget {
  final Future<bool> Function(String) checkQRCode;
  const QrCodeScreen({
    Key? key,
    required this.checkQRCode,
  }) : super(key: key);

  @override
  QrCodeScreenState createState() => QrCodeScreenState();
}

class QrCodeScreenState extends State<QrCodeScreen>
    with SingleTickerProviderStateMixin {
  late final AnimationController _controller;
  final ValueNotifier<bool> scanningValue = ValueNotifier(true);
  final QRRepository repo = QRRepository();

  MobileScannerController cameraController = MobileScannerController();

  @override
  void initState() {
    super.initState();
    _controller =
        AnimationController(vsync: this, duration: const Duration(seconds: 2))
          ..repeat();
  }

  void _pause() {
    // cameraController.stop();
    _controller.stop();
    scanningValue.value = false;
  }

  void _resume() {
    cameraController.start();
    _controller.repeat();
    scanningValue.value = true;
  }

  void _onCheck(code) async {
    setState(() {
      isLoading = true;
    });
    await Future.delayed(const Duration(seconds: 1));
    isDone = await widget.checkQRCode(code);
    setState(() {});
    Timer(const Duration(milliseconds: 2500), () {
      if (mounted) {
        Get.back();
      }
    });
  }

  bool? isDone;
  bool? isLoading;

  @override
  Widget build(BuildContext context) {
    return Scaffold(
      body: GestureDetector(
        onTap: () {
          if (!scanningValue.value) _resume();
        },
        child: Stack(
          children: [
            Container(
              width: Get.width,
              height: Get.height,
              child: MobileScanner(
                allowDuplicates: false,
                controller: cameraController,
                onDetect: (barcode, args) async {},
              ),
            ).blurred(
              blurColor: Colors.white10,
              colorOpacity: .1,
              overlay: Center(
                child: ClipRRect(
                  borderRadius: BorderRadius.circular(8),
                  child: SizedBox(
                    height: Get.width * .9,
                    width: Get.width * .9,
                    child: Stack(
                      children: [
                        MobileScanner(
                          allowDuplicates: false,
                          controller: cameraController,
                          onDetect: (barcode, args) async {
                            if (barcode.rawValue == null) {
                              debugPrint('Failed to scan Barcode');
                            } else {
                              final String code = barcode.rawValue!;
                              debugPrint('Barcode found! $code');
                              HapticFeedback.heavyImpact();
                              try {
                                if (isLoading != true) {
                                  _pause();
                                  _onCheck(code);
                                }
                              } catch (ex) {
                                appDebugPrint('Error scan" $ex');
                              }
                            }
                          },
                        ),
                        ValueListenableBuilder<bool>(
                          valueListenable: scanningValue,
                          builder: (_, value, child) {
                            if (isDone == true) {
                              return Center(
                                child: WidgetGlassBackground(
                                  borderRadius: BorderRadius.circular(20),
                                  padding: const EdgeInsets.all(24),
                                  child: Column(
                                    mainAxisSize: MainAxisSize.min,
                                    children: [
                                      Text(
                                        'Approved success',
                                        style: robotoMedium.copyWith(
                                            fontSize: 20, color: Colors.black),
                                      ),
                                      const SizedBox(
                                        height: 24,
                                      ),
                                      Lottie.asset(assetlottie('success'),
                                          package: 'oowoo_core',
                                          repeat: false,
                                          width: Get.width / 3),
                                    ],
                                  ),
                                ),
                              );
                            } else if (isDone == false) {
                              return Center(
                                child: WidgetGlassBackground(
                                  borderRadius: BorderRadius.circular(20),
                                  padding: const EdgeInsets.all(24),
                                  child: Column(
                                    mainAxisSize: MainAxisSize.min,
                                    children: [
                                      Text(
                                        'This qrcode is not valid',
                                        style: robotoMedium.copyWith(
                                            fontSize: 20, color: Colors.black),
                                      ),
                                      const SizedBox(
                                        height: 24,
                                      ),
                                      Lottie.asset(assetlottie('qrError'),
                                          package: 'oowoo_core',
                                          repeat: false,
                                          width: Get.width / 3),
                                    ],
                                  ),
                                ),
                              );
                            }
                            if (value) {
                              return child!;
                            } else {
                              return Center(
                                child: WidgetGlassBackground(
                                  borderRadius: BorderRadius.circular(20),
                                  padding: const EdgeInsets.all(24),
                                  child: Column(
                                    mainAxisSize: MainAxisSize.min,
                                    children: [
                                      Text(
                                        'Sending your request',
                                        style: robotoMedium.copyWith(
                                            fontSize: 20, color: Colors.black),
                                      ),
                                      const SizedBox(
                                        height: 24,
                                      ),
                                      Lottie.asset(assetlottie('checkingQr'),
                                          package: 'oowoo_core',
                                          width: Get.width / 3),
                                    ],
                                  ),
                                ),
                              );
                            }
                          },
                          child: Center(
                            child: Lottie.asset(assetlottie('scan'),
                                package: 'oowoo_core',
                                repeat: true,
                                controller: _controller),
                          ),
                        ),
                      ],
                    ),
                  ),
                ),
              ),
            ),
            Align(
              alignment: Alignment.bottomCenter,
              child: Padding(
                  padding: EdgeInsets.only(
                      bottom: MediaQuery.of(context).padding.bottom + 32),
                  child: InkWell(
                    onTap: () {
                      Get.back();
                    },
                    child: WidgetGlassBackground(
                      borderRadius: BorderRadius.circular(8),
                      padding: const EdgeInsets.symmetric(
                          horizontal: 24, vertical: 8),
                      child: Text(
                        'Cancel',
                        style: robotoBold.copyWith(
                            fontSize: 20, color: Colors.black),
                      ),
                    ),
                  )),
            )
          ],
        ),
      ),
    );
  }

  @override
  void dispose() {
    scanningValue.dispose();
    cameraController.dispose();
    _controller.dispose();
    super.dispose();
  }
}
