import 'dart:async';

import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';
import 'package:intl/intl.dart';
import '../widget_bottom_sheet_base.dart';

class DateTimeFormat {
  String format;
  String? _label;

  String get label {
    _label ??= DateFormat(format).format(DateTime.now());
    return _label!;
  }

  DateTimeFormat({required this.format});
}

class DateTimeFormatWidget extends StatefulWidget {
  final dynamic dateFormat;
  final dynamic timeFormat;
  final dynamic onSelectedItemChanged;
  final dynamic onTimeFormat;
  final dynamic selectedColor;
  final dynamic unselectedColor;
  final dynamic style;
  const DateTimeFormatWidget({
    Key? key,
    required this.dateFormat,
    required this.timeFormat,
    required this.onSelectedItemChanged,
    required this.onTimeFormat,
    required this.selectedColor,
    required this.unselectedColor,
    required this.style,
  }) : super(key: key);

  @override
  _DateTimeFormatWidgetState createState() => _DateTimeFormatWidgetState();
}

class _DateTimeFormatWidgetState extends State<DateTimeFormatWidget> {
  late dynamic timeFormat = widget.timeFormat;
  late dynamic dateFormat = widget.dateFormat;

  List<DateTimeFormat> timeFormats = [
    DateTimeFormat(format: 'H:mm'),
    DateTimeFormat(format: 'hh:mm aaa')
  ];

  List<DateTimeFormat> dateFormats = [
    DateTimeFormat(format: 'dd MMMM yyyy'),
    DateTimeFormat(format: 'MMMM dd, yyyy'),
    DateTimeFormat(format: 'dd MMMM yy'),
    DateTimeFormat(format: 'dd/MM/yy'),
    DateTimeFormat(format: 'yy/MM/dd'),
  ];

  bool isPressing = false;
  DateTime _time = DateTime.now();
  late FixedExtentScrollController scrollController =
      FixedExtentScrollController(
          initialItem:
              dateFormats.indexWhere((e) => e.format == widget.dateFormat));

  @override
  Widget build(BuildContext context) {
    return WidgetBottomBarApp(
      padding: const EdgeInsets.only(top: 56, left: 40, right: 32, bottom: 40),
      child: Column(
        mainAxisSize: MainAxisSize.min,
        children: [
          Row(
            mainAxisAlignment: MainAxisAlignment.spaceAround,
            children: timeFormats.map((e) {
              bool selected = timeFormat == e;
              return InkWell(
                onTap: () {
                  if (!selected) {
                    widget.onTimeFormat(e.format);
                    timeFormat = e.format;

                    setState(() {});
                  }
                },
                child: AnimatedContainer(
                  duration: const Duration(milliseconds: 300),
                  padding: const EdgeInsets.symmetric(
                      vertical: 10.0, horizontal: 24),
                  decoration: BoxDecoration(
                    borderRadius: BorderRadius.circular(8),
                    color: selected
                        ? widget.selectedColor
                        : widget.unselectedColor,
                  ),
                  child: Text(
                    e.label,
                    style: widget.style,
                  ),
                ),
              );
            }).toList(),
          ),
          const Divider(
            height: 80,
          ),
          SizedBox(
            width: 200,
            height: 180,
            child: GestureDetector(
              onTapDown: (_) {
                setState(() {
                  _time = DateTime.now();
                  isPressing = true;
                  Timer(const Duration(seconds: 1), () {
                    if (DateTime.now().difference(_time).inMilliseconds > 850 &&
                        mounted) {
                      isPressing = false;
                      setState(() {});
                    }
                  });
                });
              },
              child: Stack(
                alignment: Alignment.center,
                children: [
                  AnimatedContainer(
                    duration: const Duration(milliseconds: 150),
                    decoration: BoxDecoration(
                        color: widget.selectedColor,
                        borderRadius: BorderRadius.circular(16)),
                    padding: isPressing
                        ? const EdgeInsets.symmetric(
                            vertical: 14.0, horizontal: 30)
                        : const EdgeInsets.symmetric(
                            vertical: 10.0, horizontal: 24),
                    width: 220,
                    height: 50,
                  ),
                  CupertinoPicker(
                    scrollController: scrollController,
                    selectionOverlay:
                        const CupertinoPickerDefaultSelectionOverlay(
                      background: Colors.transparent,
                      capEndEdge: false,
                      capStartEdge: false,
                    ),
                    itemExtent: 60,
                    onSelectedItemChanged: (int index) {
                      dateFormat = dateFormats[index].format;
                      widget.onSelectedItemChanged(dateFormats[index].format);
                      _time = DateTime.now();
                      if (!isPressing) isPressing = true;
                      setState(() {});
                      Timer(const Duration(seconds: 1), () {
                        if (DateTime.now().difference(_time).inMilliseconds >
                                850 &&
                            mounted) {
                          isPressing = false;
                          setState(() {});
                        }
                      });
                    },
                    backgroundColor: Colors.transparent,
                    children: dateFormats
                        .map(
                          (e) => Center(
                            child: Text(
                              e.label,
                              style: widget.style,
                            ),
                          ),
                        )
                        .toList(),
                  ),
                ],
              ),
            ),
          ),
        ],
      ),
    );
  }
}
